// EAllocator.h: interface for the EAllocator class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EALLOCATOR_H__FD31CFC3_BB54_11D2_91EF_004005446EE6__INCLUDED_)
#define AFX_EALLOCATOR_H__FD31CFC3_BB54_11D2_91EF_004005446EE6__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

/* fun stuff notes:
   run into a really strange thing : How to call a pure virtual function and
   instantiate the object !!!
   Example:
   class A 
   {
     virtual ~A() { destructAll(); }  // call a pure virtual fn
                                      // Notes that the compiler should reject this !
     virtual void destructAll() =0;   // pure virtual fn
   };
   
   class B : public A
   {
     void destructAll() {}
   }
 */

/*! Macro used to declare a alias template class for a given allocator.
 * @param ClassAllocMethod Allocation method: EDynAllocatorT or ESimpleAllocatorT.
 * @param ClassAllocObjecTtype Type of object: EAllocatorBinT, EAllocatorObjT...
 * @param ClassName: template name. The naming convention is as follow:
 *                   ClassAllocMethod followed by type. For example a
 *                   dynamic allocator on pointer template would be:
 *                   EDynAllocatorPtrT.
 */
#define EDECLARE_ALLOCATOR( ClassAllocMethod, ClassAllocObjectType, ClassName ) \
  template<class TElm>                                                          \
  class ClassName : public ClassAllocMethod< TElm,ClassAllocObjectType<TElm> >  \
  {                                                                             \
  };



/*! \brief This class provides allocation for a vector of object.
 */
template<class TElm>
class EAllocatorT
{
public:
  typedef EAllocatorT<TElm> EAllocator;

	/*! Constructs the allocator.
   */
  EAllocatorT() {}

  /*! Destructs the allocator.
   */
  virtual ~EAllocatorT() {}

  /*! Set the ownership flag.
   *
   * Does nothing !
   *
   * @param bOwnObject If \a bOwnObject is \c true then the allocator owns the
   *                   object. This means they are destroyed when destruct() is called.
   *                   If \a bOwnObject is \c false then object are not destroyed.
   */
  virtual void setOwnership( bool bOwnObject = true ) {}

  /*! Tests if the allocator owns the object.
   * 
   * @return Always return \c true.
   */
  virtual bool ownership() const;

  /*! Get a reference on a object of a given index.
   * @param nIndex Zero based index of the object.
   * @return Reference on the object.
   */
  virtual const TElm &at( SL nIndex ) const =0;

  /*! Get a reference on a object of a given index.
   * @param nIndex Zero based index of the object.
   * @return Reference on the object.
   */
  virtual TElm &at( SL nIndex ) =0;

  /*! Initializes an object of the array.
   *
   * @param nIndex Zero based index of the object to initialize.
   */
  virtual void construct( SL nIndex ) =0;

  /*! Initializes a sequence of object in the array.
   *
   * @param nIndex Zero based index of the first object to initialize.
   * @param nLen Number of object to initialize.
   */
  virtual void construct( SL nIndex,
                          SL nLen );

  /*! Destructs an object of the array.
   * @param nIndex Zero based index of the object to destruct.
   */
  virtual void destruct( SL nIndex ) =0;

  /*! Destructs a sequence list of object in the array.
   * @param nIndex Zero based index of the first object to destruct. 
   * @param nLen Number of object to destruct. May be 0.
   */
  virtual void destruct( SL nIndex, 
                         SL nLen );

  /*! Copy a sequence of object from this array to another one.
   * @param nIndexSource Zero based index of the first object of the sequence to copy.
   * @param nLen Number of objects in the sequence. May be zero.
   * @param nIndexDestination Zero based index of destination array where the sequence
   *                          is copied to.
   * @param allocatorDest Destination array. Must be different from (*this).
   *                      Use copyToThis() otherwise.
   */
  virtual void copyTo( SL nIndexSource, 
                       SL nLen,
                       SL nIndexDestination,
                       EAllocator &allocatorDest ) const =0;

  /*! Copy a sequence of object from a standard C array to this array.
   * @param arraySource Pointer on the first element of the source array to copy.
   * @param nLen Number of objects in the sequence. May be zero.
   * @param nIndexDestination Zero based index where the sequence is copied to.
   */
  virtual void copyFrom( const TElm *arraySource, 
                         SL nLen,
                         SL nIndexDestination ) =0;

  /*! Move a sequence of object inside the array.
   * @param nIndex Zero based index of the first element to move.
   * @param nLen Number of elements to move.
   * @param nIndexDestination Zero based index where the first element is moved to.
   */
  virtual void copyToThis( SL nIndex,
                           SL nLen,
                           SL nIndexDestination ) =0;

  /*! Copy another allocator to this one.
   * @param allocator Allocator to copy.
   */
  virtual void copy( const EAllocator &allocator );

  /*! Resizes the array so that a given number of elements fit in.
   * @param nNewSize Number of elements that should fit in the array.
   *                 If 0 then the array is freeed.
   */
  virtual void resize( SL nNewSize ) =0;

  /*! Returns the exact number of elements in the array.
   * @return Number of elements in the array.
   */
  virtual SL count() const =0;

  /*! Get the allocated buffer size
   *
   * Default returns count().
   *
   * @return The maximum number of elements that can fit in the array without resizing.
   */
  virtual SL maxCount() const;

private:
  /*! Disable copy constructor.
   */
  EAllocatorT( const EAllocatorT &allocator ) {}

  /*! Disable copy operator.
   */
  void operator =( const EAllocatorT &allocator ) {}
};


template<class TElm>
void 
EAllocatorT<TElm>::construct( SL nIndex,
                              SL nLen )
{
  EASSERT( nLen >= 0 );
  while ( nLen-- )
    construct( nIndex++ );
}


template<class TElm>
void 
EAllocatorT<TElm>::destruct( SL nIndex, 
                             SL nLen )
{
  EASSERT( nLen >= 0 );
  while ( nLen-- )
    destruct( nIndex++ );
}


template<class TElm>
inline SL 
EAllocatorT<TElm>::maxCount() const
{
  return count();
}


template<class TElm>
void 
EAllocatorT<TElm>::copy( const EAllocator &allocator )
{
  SL nCount = allocator.count();
  resize( nCount );
  allocator.copyTo( 0, nCount,
                    0,
                    *this );
}


template<class TElm>
bool 
EAllocatorT<TElm>::ownership() const 
{ 
  return true; 
}

#endif // !defined(AFX_EALLOCATOR_H__FD31CFC3_BB54_11D2_91EF_004005446EE6__INCLUDED_)
