// EDynArrayT.h: interface for the EDynArrayT class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EDYNARRAYT_H__F247C3B4_9454_11D2_AF24_AA0DE873AB61__INCLUDED_)
#define AFX_EDYNARRAYT_H__F247C3B4_9454_11D2_AF24_AA0DE873AB61__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000


/* History
 * 29/03/98: - Add granularity().
 *           - Add setGranularity().
 *           - Add _nGranularity.
 *           - Change resize() to use _nGranularity.
 *           - Change init() to initialize _nGranularity.
 * 24/07/98: - Change to take in account the new template.
 * 15/12/98: - Template definition. No more depend on the EArray class.
 *             The parent class is THE template parameter.
 *           - Template renamed EDynArrayT. 
 *           - New template class created: EDynArray<Telm,isObject> for compatibly.
 */


// Types:
// #####################Last change: 21/08/97 20h00mn#########################

/*!
 * \brief Template class that provides efficient arrays of either objects or simple types.
 * 
 * The template definition is as follow:
 * \code
 * template<class TElm,bool isObject =false>
 * class EDynArray;
 * \endcode
 * 
 * This class template should be use instead of EArray if there is a large
 * number of elements in the array or there is a lot of operations that
 * add or remove element to the array.
 * 
 * The method used to minimize reallocation is to allocate a more
 * important size that the one really needed. To know the size that
 * will be allocated, a \e granularity parameter is used. The default
 * granularity is set to 8.
 *
 * The size allocated is the first multiple of \e granularity that is
 * equal or greater than the size required.
 *
 * Though the array a greater size than required, count() still returns the
 * required size. To know the allocated size, you should use maxSize().
 * 
 * For example, if resize( 5 ) is called and the granularity is 8, the
 * array will be resized to a size of 8 elements.
 *
 * You might want to set the granularity to greater value if a 
 * large amount of elements are added.
 * 
 * \sa granularity(), setGranularity().
 */
template<class ESuperClass>
class EDynArrayT : public ESuperClass
{
public:
  /*! Default constructor.
   */
  EDynArrayT();

  /*!
   * Constructs an array with a size to store \a size element.
   * @param size initial size of the array
   */
  EDynArrayT( SL size );

  /*! Destructor.
   */
  virtual ~EDynArrayT();

  /*!
   * Return the number of allocated elements for the array. This size can be
   * greater than the number of element in the array if the array use some
   * special allocation strategy when resizing (EDynaArray for example).
   * The default return the size() of the array.
   * @return size of the number of allocated element in the array.
   */
  SL maxSize() const;

  /*! Resizes the array.
   * Shrinks or expands the array so it can hold \a size elements. The new
   * elements are unitialised. If a size of 0 is given then the array is
   * empty and null.
   * @param nSize new size of the array.
   * @param bChangeCount if the array is enlarged and changeCount is true,
   *        then the number of elements in the array is changed. If the new
   *        array size is then than the current the number of elementsn then
   *        the count is changed.
   */
  void resize( SL nSize, 
               bool bChangeCount = true );

  /*! Minimize array size.
   * Reduce the size of the array, so it can holds a maximum of  size()
   * elements. This is used to reduce memory used.
   */
  void shrink();

  /*! Get granularity.
   * @return Current granularity.
   * \sa resize() 
   */
  SL granularity();

  /*! Set granularity.
   * @param nGranularity New granularity (must be > 0).
   * \sa resize()
   */
  void setGranularity( SL nGranularity );

protected:
  /*! Initialise members.
   */
  void init( SL nSize );

private:
  /*! initialise member.
   */
  void init();

  /*! Set the number of allocated element in the array.
   * Set the number of allocated element in the array.
   * @param maxSize number of allocated element in the array.
   */
  void setMaxSize( SL nMaxSize );

  /*! Real number of elements in the array.
   */
  SL _nMaxSize;

  /*! Granularity to increment array size
   */
  SL _nGranularity;
};

// Implementation:
// #####################Last change: 21/08/97 22h40mn#########################

template<class ESuperClass>
inline void 
EDynArrayT<ESuperClass>::init( SL nSize )
{
  init();
  ESuperClass::init( nSize );
}


template<class ESuperClass>
inline void 
EDynArrayT<ESuperClass>::init()
{
  _nMaxSize = 0;
  _nGranularity = 8;
}


template<class ESuperClass>
inline 
EDynArrayT<ESuperClass>::EDynArrayT()
{
  init();
}


template<class ESuperClass>
EDynArrayT<ESuperClass>::EDynArrayT( SL nSize )
{
  init();
  ESuperClass::init( nSize );
}


template<class ESuperClass>
inline 
EDynArrayT<ESuperClass>::~EDynArrayT()
{
}


template<class ESuperClass>
inline SL 
EDynArrayT<ESuperClass>::maxSize() const
{
  return _nMaxSize;
}


template<class ESuperClass>
void 
EDynArrayT<ESuperClass>::resize( SL nLen, 
                                bool bChangeCount )
{
  EASSERT( nLen >= 0 );

  SL nNewLen = ((nLen / _nGranularity) +1) * _nGranularity;

  if ( nNewLen > maxSize() )   // buffer is bigger !
    ESuperClass::resize( nNewLen, false );  // this call setMaxSize

  if ( bChangeCount )
    setCount( nLen );   // set the new number of element
}


template<class ESuperClass>
inline void 
EDynArrayT<ESuperClass>::shrink()
{
  ESuperClass::resize( count() );
}


template<class ESuperClass>
inline void 
EDynArrayT<ESuperClass>::setMaxSize( SL nMaxSize )
{
  _nMaxSize = nMaxSize;
}


template<class ESuperClass>
inline SL 
EDynArrayT<ESuperClass>::granularity()
{
  return _nGranularity;
}


template<class ESuperClass>
void 
EDynArrayT<ESuperClass>::setGranularity( SL nGranularity )
{
  EASSERT( nGranularity > 0 );
  _nGranularity = nGranularity;
}




#endif // !defined(AFX_EDYNARRAYT_H__F247C3B4_9454_11D2_AF24_AA0DE873AB61__INCLUDED_)
