// ###########################################################################
// File: EHVALITE.HPP
// (c)Copyright 03/1998, Lepilleur Baptiste (Aka Elyon from Zuul Design)
// ###########################################################################


#ifndef EHVALITE_HPP_INCLUDED


/* History:
 * 13/09/1998: - Bugfix: Using EHash in Iterator instead of EHashVal !
 * 26/10/1998: - Added: - Default constructor for Iterator.
 *                      - attach() to attach the iterator to a hash table. 
 *                      - secures calls to other methods by check that the iterator
 *                        is attached.
 *                      - count() to get the number of items.
 * 21/03/1999: - Changes: constructor & attach() now use a reference on the
 *                        hash table instead of a pointer.
 *                        ==> COMPATIBILITY BREAK.
 */

// Includes:
// #####################Last change: 28/03/98 20h01mn#########################
#include "EIteratorT.h"

// Constantes:
// #####################Last change: 28/03/98 20h01mn#########################


// Macros:
// #####################Last change: 28/03/98 20h01mn#########################


// Types:
// #####################Last change: 28/03/98 20h01mn#########################

/** Hash Table Iterator.
 * This class is used to walk through all the association in a hash table.
 * Key Type of the key, for storage and return value.
 * Item Type of the item associated with a key, for storage & return value.
 * KeyArg and ItemArg are the as Key & Item excepts that they are used when
 * passing paramaters (usually it is a reference on Key and Item type).
 *
 * Here is an example:
 *   class HashIt : public EHashValIterator<char*,UL,char*,UL>
 *   {
 *   public:
 *     HashBakaIt( HashWordFrq &hashTable ) :
 *         EHashValIterator<char*,UL,char*,UL>(&hashTable) {}
 *   };
 *
 *   This iterator works with the example of EHash classes.
 *   You can use it to print all the words in the hash table.
 *
 *   HashIt i( hash );
 *
 *   cout << "Number of Items: " << hash.count() << "\n";
 *   for ( i.begin(); !i.isEnd(); i.next() )
 *   {
 *     cout << i.key() << ": " << i.item() << "\n";
 *   }
 */
template<class Key, class Item, class KeyArg, class ItemArg>
class EHashValIterator : public EIteratorT<Item>
{
public:
  EHashValIterator( const EHashVal<Key,Item,KeyArg,ItemArg> &hashTable );
  EHashValIterator();

  void attach(  const EHashVal<Key,Item,KeyArg,ItemArg> &hashTable );

  /** Set the iterator at the beginning of the table.
   * If there is no element in the table then isEnd() will return true.
   */
  void begin();

  /** Test if the iterator is beyond the last association.
   */
  bool isEnd() const;

  /** Remove the association at the current position of the iterator.
   */
  void remove();

  /** Move the iterator to the next association.
   */
  void next();

  /** Get the item at the current position of the iterator.
   * @return item at the current position of the iterator.
   */
  Item& item();

  /** Get the item at the current position of the iterator.
   * @return item at the current position of the iterator.
   */
  const Item& item() const;

  /** Get the key at the current position of the iterator.
   * @return key at the current position of the iterator.
   */
  const Key& key() const;

  /*! Get the number of items.
   * @return Number of items.
   */
  SL count() const;

private:
  /** Search the next index with an association.
   * The search begins at the index _nIndex +1.
   */
  void searchNextIndex();

  /// Hash table the iterator operate on.
  const EHashVal<Key,Item,KeyArg,ItemArg> *_pHashTable;

  /// Current index in association table.
  SL _nIndex;

  /// Current Association for the list at nIndex.
  typename EHashVal<Key,Item,KeyArg,ItemArg>::ELCAsso *_pAsso;
};


// Template implementation:
// #####################Last change: 28/03/98 20h01mn#########################

template<class Key, class Item, class KeyArg, class ItemArg>
EHashValIterator<Key,Item,KeyArg,ItemArg>::EHashValIterator(
    const EHashVal<Key,Item,KeyArg,ItemArg> &hashTable )
{
  _pHashTable = &hashTable;
  _nIndex = 0;
  _pAsso = ENIL;
  begin();
}


template<class Key, class Item, class KeyArg, class ItemArg>
EHashValIterator<Key,Item,KeyArg,ItemArg>::EHashValIterator()
{
  _pHashTable = NULL;
  _nIndex = 0;
  _pAsso = ENIL;
}


template<class Key, class Item, class KeyArg, class ItemArg>
void EHashValIterator<Key,Item,KeyArg,ItemArg>::attach(  
    const EHashVal<Key,Item,KeyArg,ItemArg> &hashTable )
{
  _pHashTable = &hashTable;
  _nIndex = 0;
  _pAsso = ENIL;
  begin();
}

template<class Key, class Item, class KeyArg, class ItemArg>
void EHashValIterator<Key,Item,KeyArg,ItemArg>::begin()
{
  EASSERT( _pHashTable );
  _nIndex = -1;
  searchNextIndex();
}

template<class Key, class Item, class KeyArg, class ItemArg>
bool EHashValIterator<Key,Item,KeyArg,ItemArg>::isEnd() const
{
  EASSERT( _pHashTable );
  return _pAsso == ENIL;
}


template<class Key, class Item, class KeyArg, class ItemArg>
void EHashValIterator<Key,Item,KeyArg,ItemArg>::remove()
{
  EASSERT( _pHashTable );
  if ( isEnd() )
    return;

  EHashVal<Key,Item,KeyArg,ItemArg>::ELCAsso *pAsso =
      _pHashTable->_aAsso[ _nIndex ];
  EHashVal<Key,Item,KeyArg,ItemArg>::ELCAsso **pPrevAsso =
      &_pHashTable->_aAsso[ _nIndex ];

  while ( pAsso != _pAsso )     // search previous association
  {
    EASSERT( pAsso );
    pPrevAsso = &pAsso->_pNext;
    pAsso = pAsso->_pNext;
  }

  _pHashTable->freeAsso( _pAsso->_key, _pAsso->_item );

  *pPrevAsso = _pAsso->_pNext;       // unchain
  delete _pAsso;
  _pAsso = *pPrevAsso;
  _pHashTable->_nNbAsso--;

  if ( !_pAsso )                  // if no more in the list for this index
  {                               // then search next.
    searchNextIndex();
  }
}

/** Move the iterator to the next association.
  */
template<class Key, class Item, class KeyArg, class ItemArg>
void EHashValIterator<Key,Item,KeyArg,ItemArg>::next()
{
  EASSERT( _pHashTable );
  if ( _pAsso )
  {
    _pAsso = _pAsso->_pNext;
    if ( !_pAsso )          // end of current index reached ?
      searchNextIndex();
  }
}

/** Search the next index with an association.
  * The search begins at the index _nIndex +1.
  */
template<class Key, class Item, class KeyArg, class ItemArg>
void EHashValIterator<Key,Item,KeyArg,ItemArg>::searchNextIndex()
{
  EASSERT( _pHashTable );
  SL nSize = _pHashTable->size();
  _nIndex++;
  for ( ;_nIndex < nSize; _nIndex++ )
  {
    if ( _pHashTable->_aAsso[ _nIndex ] )
    {
      _pAsso = _pHashTable->_aAsso[ _nIndex ];
      return;
    }
  }
  _pAsso = ENIL;
}

template<class Key, class Item, class KeyArg, class ItemArg>
const Key& EHashValIterator<Key,Item,KeyArg,ItemArg>::key() const
{
  EASSERT( _pHashTable );
  EASSERT( _pAsso );
  return _pAsso->_key;
}

template<class Key, class Item, class KeyArg, class ItemArg>
const Item& EHashValIterator<Key,Item,KeyArg,ItemArg>::item() const
{
  EASSERT( _pHashTable );
  EASSERT( _pAsso );
  return _pAsso->_item;
}

template<class Key, class Item, class KeyArg, class ItemArg>
Item& EHashValIterator<Key,Item,KeyArg,ItemArg>::item()
{
  EASSERT( _pHashTable );
  EASSERT( _pAsso );
  return _pAsso->_item;
}


template<class Key, class Item, class KeyArg, class ItemArg>
inline SL 
EHashValIterator<Key,Item,KeyArg,ItemArg>::count() const
{
  EASSERT( _pHashTable );
  return _pHashTable->count();
}


// Prototypes:
// #####################Last change: 28/03/98 20h01mn#########################



#define EHVALITE_HPP_INCLUDED
#endif


// ########################### EOF EHVALITE.HPP ##############################
// ########################### EOF EHVALITE.HPP ##############################
// ########################### EOF EHVALITE.HPP ##############################

