// ERect.h: interface for the ERect class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_ERECT_H__180BBAEB_91B8_11D2_AF19_93F5DFEBAA61__INCLUDED_)
#define AFX_ERECT_H__180BBAEB_91B8_11D2_AF19_93F5DFEBAA61__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "EVector2.h"
#include "ESize.h"

template<class ERectVector,class ERectSize>
class ERectT  
{
public:
  /*! Constructs a null rectangle.
   */
	ERectT();

  /*! Constructs a rectangle from its corner coordinate.
   * @param v2Corner1 Coordinate of the first corner.
   * @param v2Corner2 Coordinate of the second corner.
   */
  ERectT( const ERectVector &v2Corner1,
          const ERectVector &v2Corner2 );

  /*! Constructs a rectangle from the 'min' corner and its size.
   * @param v2Min Coordinate of its minimum corner coordinate.
   * @param sizeRect Size of the rectangle.
   */
  ERectT( const ERectVector &v2Corner,
          const ERectSize &sizeRect );

  /*! Destructor.
   */
  virtual ~ERectT() {};

  /*! Set rectangle to the null rectangle.
   * @return Reference on this object.
   */
  ERectT<ERectVector,ERectSize> &setNull();

  /*! Constructs a rectangle from its corner coordinate.
   * @param v2Corner1 Coordinate of the first corner.
   * @param v2Corner2 Coordinate of the second corner.
   * @return Reference on this object.
   */
  ERectT<ERectVector,ERectSize> &setRect( const ERectVector &v2Corner1,
                                          const ERectVector &v2Corner2 );

  /*! Constructs a rectangle from the 'min' corner and its size.
   * @param v2Min Coordinate of its minimum corner coordinate.
   * @param sizeRect Size of the rectangle.
   * @return Reference on this object.
   */
  ERectT<ERectVector,ERectSize> &setRect( const ERectVector &v2Corner,
                                          const ERectSize &sizeRect );

  /*! Set min corner.
   * @param v2MinCorner New min corner.
   * @return Reference on this object.
   */
  ERectT<ERectVector,ERectSize> &setCorner( const ERectVector &v2MinCorner );

  /*! Set size.
   * @param sizeRect New size.
   * @return Reference on this object.
   */
  ERectT<ERectVector,ERectSize> &setSize( const ERectSize &sizeRect );

  /*! Inflate the rectangle.
   * Move the min corner by \a -v2Inflate and the max corner by \a v2Inflate.
   * @param v2Inflate Inflate vector.
   */
  ERectT<ERectVector,ERectSize> &inflate( const EVector2 &v2Inflate );

  /*! Deflate the rectangle.
   * Move the min corner by \a v2Deflate and the max corner by \a -v2Deflate.
   * @param v2Deflate Deflate vector.
   */
  ERectT<ERectVector,ERectSize> &deflate( const EVector2 &v2Deflate );

  /*! Get the min corner.
   * @return Min corner.
   */
  const ERectVector &corner() const;

  /*! Get the size.
   * @return Size.
   */
  const ERectSize &size() const;

  /*!Get the center of the rectangle.
   * @return Center of the rectangle.
   */
  ERectVector center() const;

private:
  /**@shapeType DependencyLink*/
  /*#  EVector2T lnkUnnamed1*/
  /**@shapeType DependencyLink*/
  /*#  ESizeT lnkUnnamed*/
  ERectVector _v2Min;
  ERectSize _sizeRect;
};


typedef ERectT<EVector2,ESize> ERect;
typedef ERectT<EVector2I,ESizeI> ERectI;



template<class ERectVector,class ERectSize>
inline ERectT<ERectVector,ERectSize> &
ERectT<ERectVector,ERectSize>::setCorner( const ERectVector &v2MinCorner )
{
  _v2Min = v2MinCorner;
  return *this;
}


template<class ERectVector,class ERectSize>
inline ERectT<ERectVector,ERectSize> &
ERectT<ERectVector,ERectSize>::setSize( const ERectSize &sizeRect )
{
  _sizeRect = sizeRect;
  return *this;
}


template<class ERectVector,class ERectSize>
inline const ERectVector &
ERectT<ERectVector,ERectSize>::corner() const
{
  return _v2Min;
}


template<class ERectVector,class ERectSize>
inline const ERectSize &
ERectT<ERectVector,ERectSize>::size() const
{
  return _sizeRect;
}


template<class ERectVector,class ERectSize>
ERectT<ERectVector,ERectSize>::ERectT()
{
  setNull();
}


template<class ERectVector,class ERectSize>
ERectT<ERectVector,ERectSize>::ERectT( const ERectVector &v2Corner1,
                                       const ERectVector &v2Corner2 )
{
  setRect( v2Corner1, v2Corner2 );
}


template<class ERectVector,class ERectSize>
ERectT<ERectVector,ERectSize>::ERectT( const ERectVector &v2Corner,
                                       const ERectSize &sizeRect )
{
  setRect( v2Corner, sizeRect );
}


template<class ERectVector,class ERectSize>
ERectT<ERectVector,ERectSize> &
ERectT<ERectVector,ERectSize>::setNull()
{
  _v2Min.setNull();
  _sizeRect.setNull();
  return *this;
}


template<class ERectVector,class ERectSize>
ERectT<ERectVector,ERectSize> &
ERectT<ERectVector,ERectSize>::setRect( const ERectVector &v2Corner1,
                                        const ERectVector &v2Corner2 )
{
  ERectSize size( eabs( v2Corner1.x() - v2Corner2.x() ) +1,
               eabs( v2Corner1.y() - v2Corner2.y() ) +1 );
  ERectVector v2Min( EMIN( v2Corner1.x(), v2Corner2.x() ),
                   EMIN( v2Corner1.y(), v2Corner2.y() ) );

  _sizeRect = size;
  _v2Min = v2Min;
  return *this;
}


template<class ERectVector,class ERectSize>
ERectT<ERectVector,ERectSize> &
ERectT<ERectVector,ERectSize>::setRect( const ERectVector &v2Corner,
                                        const ERectSize &sizeRect )
{
  _sizeRect = sizeRect;
  _v2Min = v2Corner;
  return *this;
}


template<class ERectVector,class ERectSize>
ERectVector
ERectT<ERectVector,ERectSize>::center() const
{
  ERectSize size = _sizeRect;
  size.scale( 0.5 );
  
  ERectVector v2Center = _v2Min;
  v2Center += size.diagonal();
  
  return v2Center;
}


template<class ERectVector,class ERectSize>
ERectT<ERectVector,ERectSize> &
ERectT<ERectVector,ERectSize>::inflate( const EVector2 &v2Inflate )
{
  _v2Min -= v2Inflate;
  _sizeRect.addToDiagonal( v2Inflate );

  return *this;
}


template<class ERectVector,class ERectSize>
ERectT<ERectVector,ERectSize> &
ERectT<ERectVector,ERectSize>::deflate( const EVector2 &v2Deflate )
{
  EVector2 v2Inc( EMIN( _sizeRect.width()/2.0, v2Deflate.x() ),
                  EMIN( _sizeRect.height()/2.0, v2Deflate.y() ) );
  _v2Min += v2Inflate();
  _sizeRect.addToDiagonal( v2Inc );

  return *this;
}


#endif // !defined(AFX_ERECT_H__180BBAEB_91B8_11D2_AF19_93F5DFEBAA61__INCLUDED_)
