// EStoreObjectReader.h: interface for the EStoreObjectReader class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_ESTOREOTREADER_H__27A1D181_7332_11D3_BE8F_0040053AD648__INCLUDED_)
#define AFX_ESTOREOTREADER_H__27A1D181_7332_11D3_BE8F_0040053AD648__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "EStoreOtReaderContext.h"
#include "EStoreOtDef.h"
#include "EStoreOtPrp.h"
class EStoreOtBuilder;


class EStoreOtReader  
{
public:
  
  typedef UL TokenType;

  enum
  {
    tktpNone =0,
    tktpObject,
    tktpObjectClose,
    tktpProperty,
    tktpError,

    tktpFirst,
  };

  enum Error
  {
    errNone =0,

    /// Expected '(' that begin an object.
    errExpectedObject,        

    /// Expected the type of the object following '('
    errExpectedObjectType,

    /// Use of unknown object type.
    errBadObjectType,

    /// Expected '=' in property definition
    errExpectedPrpEqual,

    /// Use of unknown property type.
    errBadPrpType,
    
    /// Expected value of the property after '='
    errExpectedPrpValue,

    /// Syntax error in property value
    errBadPrpValue,

    /// File ended, while some objects have not been closed.
    errUncloseObject,

    /// Unexpected characters after ')'
    errSyntax,

    /// End of Stream
    errEof,

    /// Bad property value type
    errBadPrpValueType,
  };

  /** Constructs a reader.
   */
	EStoreOtReader();

  /** Destructor.
   */
	virtual ~EStoreOtReader();

  /** Set the object definition use by the reader.
   * @param def Reference on the object definition used by the reader.
   */
  void setObjectDef( EStoreOtDef &def );

  /** Read the stream.
   * 
   * Initialize the parsing and call doRead(). The method doRead() should then
   * parse the stream using the readToken() method.
   */
  void read( EDataStream &streamIn );

  /** Read a token from the stream.
   * @return Type of the token read.
   */
  TokenType readToken();

  /** Implement the reading of the stream.
   * Should be overrided.
   */
  virtual void doRead() {}

protected:
  /** Return the stream data are read from.
   * @return Reference on the datastream.
   */
  EDataStream &stream();

  /*! Read a byte from the input stream.
   *
   * Read a character from the input stream and convert dos, unix & mac line return
   * to a simple '\n'.
   *
   * @return Byte read from the input stream.
   */
  UB readByte();

  /** Read a line from the input stream.
   * @param strLine [Out] Line read.
   */
  void readLine( EString &strLine );

  EStoreOtDef &def();

  TokenType makeTokenError( Error err );

  TokenType makeTokenObject( EString &strLine );

  TokenType makeTokenPrp( EString &strLine );

protected:
  /** Reader context.
   */
  EStoreOtReaderContext _context;

  /** Error token information. 
   */
  Error _errToken;

  /** Object type of the token.
   */
  EStoreObjectType _esotpToken;

  /** Property value of the token.
   */
  EStoreOtPrp _prpToken;

private:
  /** Pointer on the datastream data are read from.
   * Set by read().
   */
  EDataStream *_pstreamData;

  /** Pointer on the definition use for reading.
   */
  EStoreOtDef *_pdefObject;

  /** Current line number since read() was called.
   */
  SL _nLineNo;

  /** Is the last char read from the stream a '\r' ?
   */
  bool _bIsCr;

  /** Has the end of the stream been reached ?
   */
  bool _bIsEndOfStream;
};




inline UB 
EStoreOtReader::readByte()
{
  UB ubChar = stream().getByte();
  
  if ( ubChar == '\r' )       // Dos CR -> indicates '\r' already parsed 
  {
    _bIsCr = true;            // so '\n' is ignored
    ubChar = '\n';
  }
  else if ( ubChar == '\n' )  // Unix CR & Dos 2nd char for CR
  {
    if ( _bIsCr )             // Dos CR already processed => ignore this one !
    {
      _bIsCr = false;          // Ignore this '\n' since '\r' as already be treated !
      ubChar = stream().getByte();
      if ( ubChar == '\r' )
      {
        _bIsCr = true;       
        ubChar = '\n';
      }
    }
  }

  if ( ubChar == '\n' )
    ++_nLineNo;

  return ubChar;
}


inline EStoreOtDef &
EStoreOtReader::def()
{
  EASSERT( _pdefObject );
  return *_pdefObject;
}


inline void 
EStoreOtReader::setObjectDef( EStoreOtDef &def )
{
  _pdefObject = &def;
}


inline EDataStream &
EStoreOtReader::stream()
{
  EASSERT( _pstreamData );
  return *_pstreamData;
}

#endif // !defined(AFX_ESTOREOTREADER_H__27A1D181_7332_11D3_BE8F_0040053AD648__INCLUDED_)
