// EVector.h: interface for the EVector class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EVECTOR_H__BB179D71_5A24_11D2_AE7A_A751456EB561__INCLUDED_)
#define AFX_EVECTOR_H__BB179D71_5A24_11D2_AE7A_A751456EB561__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000



#ifdef EDBG
#define EVECTOR_SECURE
#endif

#ifdef EVECTOR_SECURE
#  define EVECTOR_CHECKVALID( v )   EASSERT( (v)._bIsValid );
#  define EVECTOR_SETVALID( v )   (v)._bIsValid = true;
#else
#  define EVECTOR_CHECKVALID( v )   
#  define EVECTOR_SETVALID( v )   
#endif

class EDataStream;

/*! \brief This templates represents a generic vector.
 * 
 * @param nDim Number of dimension of the vector.
 * @param EGmtVal Value type.
 */
template<SL nDim,class EGmtVal>
class EVector  
{
public:
  typedef EVector<nDim,EGmtVal> EVectorN;

	EVector();
  
  /*! Copy contrustor.
   * @param v Vector to copy.
   */
  EVector( const EVectorN &v );

  /*! Destructor.
   */
  ~EVector();

  /*! Copy a vector.
   * @param v Vector to copy.
   * @return Reference on this vector.
   */
  EVectorN &copy( const EVectorN &v );

  /*! Assigment operator.
   * @param v Vector to copy.
   * @return Reference on this vector.
   */
  EVectorN &operator =( const EVectorN &v );

  /*! Set a coordinate of a vector.
   * @param nCoord Zero based index of the coordinate to set.
   * @param fValue Coordinate value.
   * @return Reference on this vector.
   */
  EVectorN &setCoord( SL nCoord,
                      EGmtVal fValue );

  /*! Set all coordinates to 0.
   * @return Reference on this vector.
   */
  EVectorN &setNull();

  /*! Get a coordinate of a vector.
   * @param nCoord Zero based index of the coordinate to retreive.
   * @return Coordinate value.
   */
  EGmtVal coord( SL nCoord ) const;

  /*! Makes this vector the sum of two other vectors.
   * @param v1 Vector 1.
   * @param v2 Vector 2.
   * @return Reference on this vector.
   */
  EVectorN &add( const EVectorN &v1,
                 const EVectorN &v2 );

  /*! Makes this vector the difference of two other vectors.
   * 
   * Resulting vector : v = v1 - v2.
   *
   * @param v1 Vector 1.
   * @param v2 Vector 2.
   * @return Reference on this vector.
   */
  EVectorN &sub( const EVectorN &v1,
                 const EVectorN &v2 );

  /*! Adds a vector to this one.
   * @param v Vector to add.
   * @return Reference on this vector.
   */
  EVectorN &operator +=( const EVectorN &v );

  /*! Subtracts a vector to this one.
   * @param v Vector to substract.
   * @return Reference on this vector.
   */
  EVectorN &operator -=( const EVectorN &v );

  /*! Scales vector coordinates.
   * @param fScale Scale factor. Each coordinates are multiplied by this factor.
   * @return Reference on this vector.
   */
  EVectorN &scale( EGmtVal fScale );

  /*! Scales vector coordinates.
   * @param vScale Scale each component of the vector by the same component of the
   *               vector \a v2Scale.
   * @return Reference on this vector.
   */
  EVectorN &scale( const EVectorN &vScale );

  /*! Copies a scaled vector.
   *
   * The resulting coordinate values are: this[i] = fScale * v[i].
   *
   * @param v Vector to scale.
   * @param fScale Scale factor applied to the other vector.
   * @return Reference on this vector.
   */
  EVectorN &linear( const EVectorN &v,
                    EGmtVal fScale );

  /*! Add a scaled vector to this one.
   *
   * The resulting coordinate values are: this[i ] += fScale * v[i].
   *
   * @param v Vector to scale.
   * @param fScale Scale factor applied to the other vector.
   * @return Reference on this vector.
   */
  EVectorN &addLinear( const EVectorN &v,
                       EGmtVal fScale );

  /*! Sub a scaled vector to this one.
   *
   * The resulting coordinate values are: this[i ] -= fScale * v[i].
   *
   * @param v Vector to scale.
   * @param fScale Scale factor applied to the other vector.
   * @return Reference on this vector.
   */
  EVectorN &subLinear( const EVectorN &v,
                       EGmtVal fScale );

  /*! Scalar products of with another vector.
   * @param v Vector.
   * @return Scalar products of this vector and v.
   */
  EGmtVal scalar( const EVectorN &v ) const;

  /*! Computes the squared scalar products of a vector.
   * 
   * The resulting value is: for i from 0 to nDim-1: Value += coord[i]*coord[i].
   *
   * Notes that the resulting value is the squared length of the vector.
   * @return Squared scalar products of the vector.
   */
  EGmtVal square() const;

  /*! Computes the length of a vector.
   * @return Length of the vector.
   */
  EGmtVal length() const;

  /*! Tests if the vector is the null vector.
   * @return \c true if the vector is the null vector.
   */
  bool isNull() const;

  /*! Normalize a vector.
   * @return Reference on this vector.
   */
  EVectorN &normalize();

  /*! Oppose vector.
   *
   * Multiply each components by -1.
   */
  EVectorN &oppose();

  /*! Save vector data.
   * @param s Stream data are saved to.
   */
  EDataStream &saveData( EDataStream &s ) const;

  /*! Load vector data.
   * @param s Stream data are loaded from.
   */
  EDataStream &loadData( EDataStream &s );
  
private:
  EGmtVal _aCoord[ nDim ];
#ifdef EVECTOR_SECURE
  bool _bIsValid;
#endif
};

/*! Computes the scalar products of two vectors.
 * @param v1 Vector 1.
 * @param v2 Vector 2.
 * @return Scalar products of v1 by v2.
 */
template<SL nDim,class EGmtVal>
EGmtVal operator *( const EVector<nDim,EGmtVal> &v1,
                    const EVector<nDim,EGmtVal> &v2 );

// Inlines
// ####################################################

// Global
// ++++++++++++++++++++++++++++++++++++++++++++++++++++
template<SL nDim,class EGmtVal>
EGmtVal operator *( const EVector<nDim,EGmtVal> &v1,
                    const EVector<nDim,EGmtVal> &v2 )
{
  EGmtVal fScalar;
  fScalar = v1.coord( 0 ) * v2.coord( 0 );
  SL n;
  for ( n =1; n < nDim; ++n )
    fScalar += v1.coord( n ) * v2.coord( n );
  return fScalar;
}


// EVector
// ++++++++++++++++++++++++++++++++++++++++++++++++++++

template<SL nDim,class EGmtVal>
EVector<nDim,EGmtVal>::EVector()
{
#ifdef EVECTOR_SECURE  
  _bIsValid = false;
#endif
}

template<SL nDim,class EGmtVal>
inline 
EVector<nDim,EGmtVal>::EVector( const EVector<nDim,EGmtVal> &v )
{
  copy ( v );
}

template<SL nDim,class EGmtVal>
EVector<nDim,EGmtVal>::~EVector()
{
}

template<SL nDim,class EGmtVal>
inline EVector<nDim,EGmtVal> &
EVector<nDim,EGmtVal>::copy( const EVectorN &v )
{
  EVECTOR_SETVALID( *this );
  SL n;
  for ( n =0; n < nDim; ++n )
    _aCoord[ n ] = v._aCoord[ n ];
  return *this;
}

template<SL nDim,class EGmtVal>
inline EVector<nDim,EGmtVal> &
EVector<nDim,EGmtVal>::operator =( const EVectorN &v )
{
  EVECTOR_SETVALID( *this );
  SL n;
  for ( n =0; n < nDim; ++n )
    _aCoord[ n ] = v._aCoord[ n ];
  return *this;
}


template<SL nDim,class EGmtVal>
inline EVector<nDim,EGmtVal> &
EVector<nDim,EGmtVal>::setCoord( SL nCoord,
                                 EGmtVal fValue )
{
  EVECTOR_SETVALID( *this );         // should check that each component has been set but...
  _aCoord[ nCoord ] = fValue;
  return *this;
}


template<SL nDim,class EGmtVal>
inline EVector<nDim,EGmtVal> &
EVector<nDim,EGmtVal>::setNull()
{
  EVECTOR_SETVALID( *this );         
  SL n;
  for ( n =0; n < nDim; ++n )
    _aCoord[ n ] = (EGmtVal)0;
  return *this;
}


template<SL nDim,class EGmtVal>
inline EGmtVal 
EVector<nDim,EGmtVal>::coord( SL nCoord ) const
{
  EVECTOR_CHECKVALID( *this );
  return _aCoord[ nCoord ];
}


template<SL nDim,class EGmtVal>
inline EVector<nDim,EGmtVal> &
EVector<nDim,EGmtVal>::add( const EVectorN &v1,
                           const EVectorN &v2 )
{
  EVECTOR_CHECKVALID( v1 );
  EVECTOR_CHECKVALID( v2 );
  EVECTOR_SETVALID( *this );
  SL n;
  for ( n =0; n < nDim; ++n )
    _aCoord[ n ] = v1._aCoord[ n ] + v2._aCoord[ n ];
  return *this;
}


template<SL nDim,class EGmtVal>
inline EVector<nDim,EGmtVal> &
EVector<nDim,EGmtVal>::sub( const EVectorN &v1,
                    const EVectorN &v2 )
{
  EVECTOR_CHECKVALID( v1 );
  EVECTOR_CHECKVALID( v2 );
  EVECTOR_SETVALID( *this );
  SL n;
  for ( n =0; n < nDim; ++n )
    _aCoord[ n ] = v1._aCoord[ n ] - v2._aCoord[ n ];
  return *this;
}


template<SL nDim,class EGmtVal>
inline EVector<nDim,EGmtVal> &
EVector<nDim,EGmtVal>::operator +=( const EVectorN &v )
{
  EVECTOR_CHECKVALID( v );
  EVECTOR_SETVALID( *this );
  SL n;
  for ( n =0; n < nDim; ++n )
    _aCoord[ n ] += v._aCoord[ n ];
  return *this;
}


template<SL nDim,class EGmtVal>
inline EVector<nDim,EGmtVal> &
EVector<nDim,EGmtVal>::operator -=( const EVectorN &v )
{
  EVECTOR_CHECKVALID( v );
  EVECTOR_SETVALID( *this );
  SL n;
  for ( n =0; n < nDim; ++n )
    _aCoord[ n ] -= v._aCoord[ n ];
  return *this;
}


template<SL nDim,class EGmtVal>
inline EVector<nDim,EGmtVal> &
EVector<nDim,EGmtVal>::scale( EGmtVal fScale )
{
  EVECTOR_CHECKVALID( *this );
  SL n;
  for ( n =0; n < nDim; ++n )
    _aCoord[ n ] *= fScale;
  return *this;
}


template<SL nDim,class EGmtVal>
inline EVector<nDim,EGmtVal> &
EVector<nDim,EGmtVal>::scale( const EVectorN &vScale )
{
  EVECTOR_CHECKVALID( *this );
  EVECTOR_CHECKVALID( vScale );
  SL n;
  for ( n =0; n < nDim; ++n )
    _aCoord[ n ] *= vScale._aCoord[ n ];
  return *this;
}


template<SL nDim,class EGmtVal>
inline EVector<nDim,EGmtVal> &
EVector<nDim,EGmtVal>::linear( const EVector<nDim,EGmtVal> &v,
                               EGmtVal fScale )
{
  EVECTOR_CHECKVALID( v );
  EVECTOR_SETVALID( *this );
  SL n;
  for ( n =0; n < nDim; ++n )
    _aCoord[ n ] = v._aCoord[ n ] * fScale;
  return *this;
}


template<SL nDim,class EGmtVal>
inline EVector<nDim,EGmtVal> &
EVector<nDim,EGmtVal>::addLinear( const EVector<nDim,EGmtVal> &v,
                                  EGmtVal fScale )
{
  EVECTOR_CHECKVALID( *this );
  EVECTOR_CHECKVALID( v );
  SL n;
  for ( n =0; n < nDim; ++n )
    _aCoord[ n ] += v._aCoord[ n ] * fScale;
  return *this;
}


template<SL nDim,class EGmtVal>
inline EVector<nDim,EGmtVal> &
EVector<nDim,EGmtVal>::subLinear( const EVector<nDim,EGmtVal> &v,
                                  EGmtVal fScale )
{
  EVECTOR_CHECKVALID( *this );
  EVECTOR_CHECKVALID( v );
  SL n;
  for ( n =0; n < nDim; ++n )
    _aCoord[ n ] -= v._aCoord[ n ] * fScale;
  return *this;
}


template<SL nDim,class EGmtVal>
inline EGmtVal 
EVector<nDim,EGmtVal>::scalar( const EVector<nDim,EGmtVal> &v ) const
{
  EGmtVal fScalar;
  fScalar = coord( 0 ) * v.coord( 0 );
  SL n;
  for ( n =1; n < nDim; ++n )
    fScalar += coord( n ) * v.coord( n );
  return fScalar;
}


template<SL nDim,class EGmtVal>
inline EGmtVal 
EVector<nDim,EGmtVal>::square() const
{
  EVECTOR_CHECKVALID( *this );
  EGmtVal fScalar = esqr( coord(0) );
  SL n;
  for ( n =1; n < nDim; ++n )
    fScalar += esqr( coord(n) );

  return fScalar;
}


template<SL nDim,class EGmtVal>
inline EGmtVal 
EVector<nDim,EGmtVal>::length() const
{
  return (EGmtVal)sqrt( square() );
}


template<SL nDim,class EGmtVal>
inline EVector<nDim,EGmtVal> &
EVector<nDim,EGmtVal>::normalize()
{
  EGmtVal fLength = length();
  if ( fLength != (EGmtVal)0.0 )
    scale( 1.0 / fLength );
  return *this;
}


template<SL nDim,class EGmtVal>
inline bool 
EVector<nDim,EGmtVal>::isNull() const
{
  SL n;
  for ( n=0; n < nDim; ++n )
    if ( coord( n ) != 0 )
      return false;
  return true;
}


template<SL nDim,class EGmtVal>
inline EVector<nDim,EGmtVal> &
EVector<nDim,EGmtVal>::oppose()
{
  SL n;
  for ( n=0; n < nDim; ++n )
    setCoord( n, -coord( n ) );
  return *this;
}


template<SL nDim,class EGmtVal>
EDataStream &
EVector<nDim,EGmtVal>::saveData( EDataStream &s ) const
{
  SL n;
  for ( n=0; n < nDim; ++n )
    s  << coord( n );
  
  return s;
}


template<SL nDim,class EGmtVal>
EDataStream &
EVector<nDim,EGmtVal>::loadData( EDataStream &s )
{
  SL n;
  for ( n=0; n < nDim; ++n )
  {
    EGmtVal fVal;
    s  >> fVal;
    setCoord( n, fVal );
  }
  
  return s;
}

#endif // !defined(AFX_EVECTOR_H__BB179D71_5A24_11D2_AE7A_A751456EB561__INCLUDED_)
