// ###########################################################################
// File: EFILE.CPP
// (c)Copyright 08/1997, Lepilleur Baptiste (Aka Elyon from Zuul Design)
// version 0.3
// ###########################################################################

/* HST:
 * 29/03/98: - Bug: bad exception cause raise on EOF in getByte().
 *           - Destructor is now virtual and call ~EIODevice().
 * 04/04/98: - mFilename is now owned.
 *           - change constructors, destructor, setFilename to duplicate and
 *             free the filename.
 *           - added assertion in readBlock() & writeBlock() so length only >=0.
 * 05/04/98: - Change exception throwing to use macro ETHROW.
 * 03/10/98: - BugFix: at( pos ), forgot to update mIndex member !!!
 * 17/18/98: - BugFix: assert( data ) in readBlock & writeBlock failed if len = 0
 *             and data == NULL.
 * 03/06/2000: - BugFix: atEnd() returns false if the index was at the end of the 
 *             file but it has not yet been read beyond the end.
 */

/* TODO:
 * - Change filename to EString.
 */

#include <io.h>
#include "usuel.hpp"
#include "efile.hpp"

// Default Constructor
EFile::EFile()
{
  init();
}

/** Open a file.
 * Open a file of name \a _name using mode \a _mode.
 * @param _name name of the file
 * @param _mode mode
 * @see EIODevice::mode()
 */
EFile::EFile( const EString &fileName ) :
    _fileName( fileName )
{
  init();
}

// Destructor
EFile::~EFile()
{
  if ( isOpen() )
    close();
  EIODevice::~EIODevice();
}

// Initialisation
void EFile::init()
{
  mFile =0;
  mLength =0;
  mIndex =0;
  resetInfo();
}

// Set the name of the file
void EFile::setName( const EString &fileName )
{
  assert( !isOpen() );
  _fileName = fileName;
}

/** Test if the file exists.
 */
bool 
EFile::exists() const
{
  return exists( _fileName );
}

/** Test if a file exists.
 * @param _name name of the file to test.
 */
bool EFile::exists( const EString &fileName )
{
#ifdef _MSC_VER
    return _access( fileName.cstr(), 0 ) == 0;
#else
    return access( fileName.cstr(), F_OK ) == 0;
#endif
}

/** Open the file.
  * Open the file of name given on construction or with \l setName() using
  * mode \a _mode.
  * @param _mode mode used to open the file.
  * @see EIODevice::flags()
  * @exception EXCIODevice raised on error
  */
void EFile::open( int _mode )
{
  assert( !isOpen() );
  assert( !_fileName.isNull() );

  init();
  setMode( _mode );

  assert( !isReadable()  ||  !isWritable() );

  const char *perm =0;
  bool retryIfFailed = false;

  if ( isReadWrite() )
  {
      if ( accessMode() & EIO_Truncate )
          perm = "w+b";
      else
      {
          perm = "r+b";
          retryIfFailed = true;
      }
  }
  else
  {
      if ( isReadable() )
          perm = "rb";
      else
          if ( isWritable() )
              perm = "wb";
  }

  mFile = fopen( _fileName.cstr(), perm);
  if ( !mFile  &&  retryIfFailed )
    mFile = fopen( _fileName.cstr(), "wb");    // file do not exist, create one
  if ( mFile )
  {
      fseek( mFile, 0, SEEK_END );
      mLength = ftell( mFile );
      fseek( mFile, 0, SEEK_SET );
      mIndex =0;

      setStatus( EIO_IsOpen );
  }
  else
      ETHROW( EXCIODevice( EXCIODevice::createError ) );
}

// Close the file.
void EFile::close()
{
    assert( isOpen() );
    fclose( mFile );
    unsetStatus( EIO_IsOpen );
    init();
}

// Flush the file.
void EFile::flush()
{
    assert( isOpen() );
    fflush( mFile );
}

// Return the size of the file
SL EFile::size() const
{
    if ( isOpen() )
        return mLength;
    else
    {
        EFile f( _fileName );
        SL sz;
        f.open( EIO_ReadOnly );   // may raise an exception
        sz = f.size();
        f.close();
        return sz;
    }
}

/** Set the index of the file.
  * @param pos position of the index in the file.
  */
void EFile::at( SL pos )
{
  assert( isOpen() );

  if ( fseek( mFile, pos, SEEK_SET ) != 0 )
    ETHROW( EXCIODevice( EXCIODevice::ioError ) );
  else
    mIndex = pos;
}


/** Read a block of byte from the device.
  * Read \a len Byte from the device and store them in the buffer \a data.
  * @param data buffer where data are stored.
  * @param len number of bytes of data to read.
  * @return number of byte of data actually read.
  * @exception EXCIODevice is case of grave error.
  */
SL EFile::readBlock( void *data, SL len)
{
  assert( !len  ||  data );
  assert( isOpen() );
  assert( isReadable() );
  assert( len >= 0 );

  SL szRead;

  szRead = fread( data, 1, len, mFile );
  mIndex += szRead;

  return szRead;
}

/** Write a block of byte on the device.
  * Write \a len Byte of buffer \a data on the device.
  * @param data buffer where data are read.
  * @param len number of bytes of data to write.
  * @exception EXCIODevice is case of grave error.
  */
void EFile::writeBlock( const void *data, SL len )
{
  assert( !len  ||  data );
  assert( isOpen() );
  assert( isWritable() );
  assert( len >= 0 );

  int szWrited = fwrite( data, 1, len, mFile );
  mIndex += szWrited;
  mLength = EMAX( mLength, mIndex );      // update file length
  if ( szWrited != len )
      ETHROW( EXCIODevice( EXCIODevice::ioError ) );
}

/** Read a byte from the stream.
  * @return byte read.
  * @exception EIODevice on error
  */
UB EFile::getByte()
{
  assert( isOpen() );
  assert( isReadable() );

  int c = getc( mFile );
  if ( c != EOF )
  {
    ++mIndex;
    return (UB)(c);
  }
  ETHROW( EXCIODevice( EXCIODevice::endOfFile ) );
#ifdef _MSC_VER
  return 0;
#endif
}

/** Write a Byte on the stream.
  * @param c byte to write.
  * @exception EIODevice on error.
  */
void EFile::putByte( UB c )
{
  assert( isOpen() );
  assert( isWritable() );

  if ( fputc( c, mFile ) == EOF )
    ETHROW( EXCIODevice( EXCIODevice::ioError ) );
  else
  {
    ++mIndex;
    mLength = EMAX( mLength, mIndex );
  }
}

/** Push-back a byte on the stream.
  * @param c byte to push-back.
  * @exception EIODevice on error.
  */
void EFile::ungetByte( UB c )
{
  assert( isOpen() );
  assert( isReadable() );
  ungetc( c, mFile );
  mIndex--;
}

