// ###########################################################################
// File: EIOFILTR.HPP
// (c)Copyright 08/1997, Lepilleur Baptiste (Aka Elyon from Zuul Design)
// Version 1.01
// ###########################################################################

/* History:
 * 26/04/98: - Fix bug when included many times.
 */

#ifndef EIOFILTR_HPP_INCLUDED
#define EIOFILTR_HPP_INCLUDED

// Includes:
// #####################Last change: 23/08/97  8h00mn#########################

#include "eiodevic.hpp"

// Constantes:
// #####################Last change: 23/08/97  8h00mn#########################


// Macros:
// #####################Last change: 23/08/97  8h00mn#########################


// Types:
// #####################Last change: 23/08/97  8h00mn#########################

class EIOFilter : public EIODevice
{
  public:
    /// Default Constructor.
    EIOFilter();

    /// Destructor.
    ~EIOFilter();

    /** Open the device.
     * Open the device using mode \a _mode.
     * @param _mode mode use to open the device
     */
    virtual void open( int _mode );

    /** Close the device.
     */
    virtual void close();

    /** Flush the device.
     * Default implementation do nothing.
     */
    virtual void flush();

    /** Return the size of the device.
     * Default implementation return 0.
     * @return size of the device.
     */
    virtual SL size() const;

    /** Return the index of the device.
     * Default implementation return 0.
     * @return index of the device.
     * @exception EXCIODevice on error.
     */
    virtual SL at() const;

    /** Set the position of the device.
     * Set the position of the device (index) to \a pos. If \a pos if beyond
     * the size of the device, then the device is extended. The extension
     * region is initialised with 0.
     * Default implementation do nothing.
     * @param pos new position of the index.
     * @exception EXCIODevice on error.
     */
    virtual void at( SL pos);

    /** Test if the device is at the end.
     * Default implementation return false;
     * @return true if the device is at the end, false otherwise.
     */
    virtual bool atEnd() const;

    /** Read a block of byte from the device.
     * Read \a len Byte from the device and store them in the buffer \a data.
     * Default implementation do nothing but return 0.
     * @param data buffer where data are stored.
     * @param len number of bytes of data to read.
     * @return number of byte of data actually read.
     * @exception EXCIODevice is case of grave error.
     */
    virtual SL readBlock( void *data, SL len);

    /** Write a block of byte on the device.
     * Write \a len Byte of buffer \a data on the device.
     * Default implementation do nothing.
     * @param data buffer where data are read.
     * @param len number of bytes of data to write.
     * @exception EXCIODevice is case of grave error.
     */
    virtual void writeBlock( const void *data, SL len );

    /** Read a byte from the stream.
     * Default implementation do nothing but return 0.
     * @return byte read.
     * @exception EIODevice on error
     */
    virtual UB getByte();

    /** Write a Byte on the stream.
     * Default implementation do nothing.
     * @param c byte to write.
     * @exception EIODevice on error.
     */
    virtual void putByte( UB c );

    /** Push-back a byte on the stream.
     * Default implementation do nothing.
     * @param c byte to push-back.
     * @exception EIODevice on error.
     */
    virtual void ungetByte( UB c );
};

// inlines:
// #####################Last change: 23/08/97  8h00mn#########################

/// Default Constructor.
inline EIOFilter::EIOFilter()
{
}

/// Destructor.
inline EIOFilter::~EIOFilter()
{
}

/** Open the device.
 * Open the device using mode \a _mode.
 * @param _mode mode use to open the device
 */
inline void EIOFilter::open( int _mode )
{
  resetInfo();
  setStatus( EIO_IsOpen );
  setMode( _mode );
}

/** Close the device.
 */
inline void EIOFilter::close()
{
  EASSERT( isOpen() );
  unsetStatus( EIO_IsOpen );
}

/** Flush the device.
 * Default implementation do nothing.
 */
inline void EIOFilter::flush()
{
}

/** Return the size of the device.
 * Default implementation return 0.
 * @return size of the device.
 */
inline SL EIOFilter::size() const
{
  return 0;
}

/** Return the index of the device.
 * Default implementation return 0.
 * @return index of the device.
 * @exception EXCIODevice on error.
 */
inline SL EIOFilter::at() const
{
  return 0;
}

/** Set the position of the device.
 * Set the position of the device (index) to \a pos. If \a pos if beyond
 * the size of the device, then the device is extended. The extension
 * region is initialised with 0.
 * Default implementation do nothing.
 * @param pos new position of the index.
 * @exception EXCIODevice on error.
 */
inline void EIOFilter::at( SL pos)
{
}

/** Test if the device is at the end.
 * Default implementation return false;
 * @return true if the device is at the end, false otherwise.
 */
inline bool EIOFilter::atEnd() const
{
  return false;
}

/** Read a block of byte from the device.
 * Read \a len Byte from the device and store them in the buffer \a data.
 * Default implementation do nothing but return 0.
 * @param data buffer where data are stored.
 * @param len number of bytes of data to read.
 * @return number of byte of data actually read.
 * @exception EXCIODevice is case of grave error.
 */
inline SL EIOFilter::readBlock( void *data, SL len )
{
  return 0;
}

/** Write a block of byte on the device.
 * Write \a len Byte of buffer \a data on the device.
 * Default implementation do nothing.
 * @param data buffer where data are read.
 * @param len number of bytes of data to write.
 * @exception EXCIODevice is case of grave error.
 */
inline void EIOFilter::writeBlock( const void *data, SL len )
{
}

/** Read a byte from the stream.
 * Default implementation do nothing but return 0.
 * @return byte read.
 * @exception EIODevice on error
 */
inline UB EIOFilter::getByte()
{
  return 0;
}

/** Write a Byte on the stream.
 * Default implementation do nothing.
 * @param c byte to write.
 * @exception EIODevice on error.
 */
inline void EIOFilter::putByte( UB c )
{
}

/** Push-back a byte on the stream.
 * Default implementation do nothing.
 * @param c byte to push-back.
 * @exception EIODevice on error.
 */
inline void EIOFilter::ungetByte( UB c )
{
}


// Prototypes:
// #####################Last change: 23/08/97  8h00mn#########################



#endif


// ########################### EOF EIOFILTR.HPP ##############################
// ########################### EOF EIOFILTR.HPP ##############################
// ########################### EOF EIOFILTR.HPP ##############################

