// ###########################################################################
// File: ESTACK.HPP
// (c)Copyright 10/1997, Lepilleur Baptiste (Aka Elyon from Zuul Design)
// Version 1.1
// ###########################################################################


#ifndef ESTACK_HPP_INCLUDED


/** HST:
 * 02/04/98: - Change the template definition. It now takes two parameters,
 *             the item type and the item parameter type. In case of conflict,
 *             the second element should be of type "const TElm&".
 *           - New class: ESmartStack, a stack that can handle a very large
 *             number of element and yet be really fast.
 * 19/08/98: - BugFix: Bad arguments on inheritance def for EFastStack !!!
 * 03/03/1999: - add TElm &top()
 *             - add TElm pop()
 */

/** TO DO:
 * - FastStack:
 *   - set resizing method: never decrease and growing grain.
 * - ListStack: implement it.
 */

// Includes:
// #####################Last change: 12/10/97 19h41mn#########################
#include "EDynArray.h"
#include "elist.hpp"

// Constantes:
// #####################Last change: 12/10/97 19h41mn#########################


// Exception:
// #####################Last change: 12/10/97 19h41mn#########################

NEWEXC( EXCStackEmpty,
    "Try to pop an element from the stack while it is empty." );


// Types:
// #####################Last change: 12/10/97 19h41mn#########################

// Abstract Stack.
// *********************Last change: 12/10/97 19h45mn*************************

/** Abstract Stack.
 * This class implement a abstract stack class.
 */
template<class TElm, class TElmArg>
class EStack
{
  public:
    /** Contructs a Fast Stack.
     * @param defaultSize number of element the stack can handle first without
     *        any resizing.
     */
    EStack() {};

    /// Default destructor
    virtual ~EStack() {}

    /** Test if the stack is empty.
     * @return true if the stack is empty.
     */
    virtual bool isEmpty() const =0;

    /** Push an element on the stack.
     * @param elm element to push on the stack.
     */
    virtual void push( TElmArg elm ) =0;

    /** Pop an element from the stack.
     * Pop an element from the stack. The Stack must not be empty.
     * @param elm poped element.
     * @exception EXCStackEmpty raise if the stack is empty.
     */
    virtual void pop( TElm *elm ) =0;

    /*! Pops an element from the stack.
     *
     * Pops an element from the stack. The Stack must not be empty.
     *
     * \return Element poped from the stack.
     * @exception EXCStackEmpty raises if the stack is empty.
     */
    virtual TElm pop() =0;

    /*! Gets the top element of the stack.
     * 
     * \return Reference on the top element of the stack.
     * \exception EXCStackEmpty raises if the stack is empty.
     */
    virtual TElm &top() =0;

    /** Clear the stack.
     * Clear the stack, therefore it is empty.
     */
    virtual void clear() =0;
};


// Fast Stack
// *********************Last change: 12/10/97 19h45mn*************************

/*! \brief Fast Stack.
 * This class is design to handle a stack using a dynamic array.
 * The element are copied using their binary representation and not the
 * copy operator.
 * @param TElm Type of elements stored in the stack.
 * @param TElmArg Type of elements when passed as argument.
 * @param bIsObject Indicate if element stored in the stack are object or if they
 *                  are binary structure. See EArray for more information.
 */
template<class TElm, class TElmArg>
class EFastStack : public EStack<TElm, TElmArg>
{
  public:
    /** Contructs a Fast Stack.
     * @param defaultSize number of element the stack can handle first without
     *        any resizing.
     */
    EFastStack( SL defaultSize =0 );

    /** Test if the stack is empty.
     * @return true if the stack is empty.
     */
    bool isEmpty() const;

    /** Push an element on the stack.
     * @param elm element to push on the stack.
     */
    void push( TElmArg elm );

    /** Pop an element from the stack.
     * Pop an element from the stack. The Stack must not be empty.
     * @param elm poped element.
     * @exception EXCStackEmpty raise if the stack is empty.
     */
    void pop( TElm *elm );

    /*! Pops an element from the stack.
     *
     * Pops an element from the stack. The Stack must not be empty.
     *
     * \return Element poped from the stack.
     * @exception EXCStackEmpty raises if the stack is empty.
     */
    virtual TElm pop();

    /*! Gets the top element of the stack.
     * 
     * \return Reference on the top element of the stack.
     * \exception EXCStackEmpty raises if the stack is empty.
     */
    virtual TElm &top();

    /** Clear the stack.
     * Clear the stack, therefore it is empty.
     */
    void clear();

    /** Minimize the size of the stack.
     * Resize the dynamic array to the number of element.
     */
    void shrink();

  private:
    EDynArray<TElm,true> mStack;
};


/** Cluster Stack.
 * This stack is design for a very large number of items. It use a list of
 * clusters to store element. A cluster is a array of a fixed size
 * containing the elements.
 * @param TElm type of element stored in the stack. TElm must have a default
 *             constructor.
 */
template<class TElm, class TElmArg>
class EClusterStack : public EStack<TElm,TElmArg>
{
public:
    /** Contructs a Cluster Stack.
     * @param nClusterSize The number of element stored in a cluster.
     *                     The greater this value is, the less memory allocations
     *                     they are.
     */
    EClusterStack( SL nClusterSize = 64 );

    /// Destructor.
    ~EClusterStack();

    /** Set the cluster size.
     * The cluster size define the number of element that are in a cluster.
     * Each time a cluster is filled, a new one is created. Therefore a large
     * value for the cluster size lower the number of memory allocation but
     * increase the memory usage.
     * @param nClusterSize number of element in a cluster.
     */
    void setClusterSize( SL nClusterSize );

    /** Get the current cluster size.
     * @return the cluster size.
     */
    SL clusterSize() const;

    /** Test if the stack is empty.
     * @return true if the stack is empty.
     */
    bool isEmpty() const;

    /** Push an element on the stack.
     * @param elm element to push on the stack.
     */
    void push( TElmArg elm );

    /** Pop an element from the stack.
     * Pop an element from the stack. The Stack must not be empty.
     * @param elm poped element.
     * @exception EXCStackEmpty raise if the stack is empty.
     */
    void pop( TElm *elm );

    /*! Pops an element from the stack.
     *
     * Pops an element from the stack. The Stack must not be empty.
     *
     * \return Element poped from the stack.
     * @exception EXCStackEmpty raises if the stack is empty.
     */
    virtual TElm pop();

    /*! Gets the top element of the stack.
     * 
     * \return Reference on the top element of the stack.
     * \exception EXCStackEmpty raises if the stack is empty.
     */
    virtual TElm &top();

    /** Clear the stack.
     * Clear the stack, therefore it is empty.
     */
    void clear();
private:
    typedef TElm* ECluster;

    /// List of clusters.
    EList<TElm> _lCluster;

    /// The current cluster.
    ECluster _pCluster;

    /// The position in the current cluster.
    SL _nIndex;

    /// The cluster size.
    SL _nClusterSize;
};



// inlines:
// #####################Last change: 12/10/97 19h16mn#########################


// EFastStack<TElm>
// *********************Last change: 12/10/97 19h46mn*************************

/** Contructs a Fast Stack.
  * @param defaultSize number of element the stack can handle first without
  *        any resizing.
  */
template<class TElm, class TElmArg>
EFastStack<TElm,TElmArg>::EFastStack( SL defaultSize ) :
  mStack( defaultSize )
{
}

/** Test if the stack is empty.
  * @return true if the stack is empty.
  */
template<class TElm, class TElmArg>
inline bool EFastStack<TElm,TElmArg>::isEmpty() const
{
  return mStack.count() == 0;
}

/** Push an element on the stack.
  * @param elm element to push on the stack.
  */
template<class TElm, class TElmArg>
inline void EFastStack<TElm,TElmArg>::push( TElmArg elm )
{
  mStack.append( elm );
}

/** Pop an element from the stack.
  * @param elm poped element.
  * @exception EXCStackEmpty raise if the stack is empty.
  */
template<class TElm, class TElmArg>
inline void EFastStack<TElm,TElmArg>::pop( TElm *elm )
{
  SL idx = mStack.count() -1;
  if ( idx < 0 )
    THROW( EXCStackEmpty() );

  *elm = mStack[ idx ];
  mStack.resize( idx );
}


template<class TElm, class TElmArg>
inline TElm 
EFastStack<TElm,TElmArg>::pop()
{
  SL idx = mStack.count() -1;
  if ( idx < 0 )
    THROW( EXCStackEmpty() );

  TElm elmTmp = mStack[ idx ];
  mStack.resize( idx );
  return elmTmp;
}


template<class TElm, class TElmArg>
inline TElm &
EFastStack<TElm,TElmArg>::top()
{
  SL idx = mStack.count() -1;
  if ( idx < 0 )
    THROW( EXCStackEmpty() );

  return mStack[ idx ];
}

/** Clear the stack.
  * Clear the stack, therefore it is empty.
  */
template<class TElm, class TElmArg>
void EFastStack<TElm,TElmArg>::clear()
{
  mStack.resize( 0 );
}

/** Minimize the size of the stack.
  * Resize the dynamic array to the number of element.
  */
template<class TElm, class TElmArg>
inline void EFastStack<TElm,TElmArg>::shrink()
{
  mStack.shrink();
}



// ESmartStack
// ***************************************************************************


/** Contructs a Cluster Stack.
  * @param nClusterSize The number of element stored in a cluster.
  *                     The greater this value is, the less memory allocations
  *                     they are.
  */
template<class TElm, class TElmArg>
EClusterStack<TElm,TElmArg>::EClusterStack( SL nClusterSize )
{
  _nClusterSize = nClusterSize;
  _pCluster = ENIL;
}

/// Destructor.
template<class TElm, class TElmArg>
EClusterStack<TElm,TElmArg>::~EClusterStack()
{
  clear();
}

/** Set the cluster size.
  * The cluster size define the number of element that are in a cluster.
  * Each time a cluster is filled, a new one is created. Therefore a large
  * value for the cluster size lower the number of memory allocation but
  * increase the memory usage.
  * @param nClusterSize number of element in a cluster.
  */
template<class TElm, class TElmArg>
void EClusterStack<TElm,TElmArg>::setClusterSize( SL nClusterSize )
{
  EASSERT( _lCluster.count() == 0 );
  EASSERT( nClusterSize > 0 );
  _nClusterSize = nClusterSize;
}

/** Get the current cluster size.
  * @return the cluster size.
  */
template<class TElm, class TElmArg>
SL EClusterStack<TElm,TElmArg>::clusterSize() const
{
  return _nClusterSize;
}

/** Test if the stack is empty.
  * @return true if the stack is empty.
  */
template<class TElm, class TElmArg>
bool EClusterStack<TElm,TElmArg>::isEmpty() const
{
  return (_nIndex == 0)  &&  _lCluster.count() <= 1;
}

/** Push an element on the stack.
  * @param elm element to push on the stack.
  */
template<class TElm, class TElmArg>
void EClusterStack<TElm,TElmArg>::push( TElmArg elm )
{
  if ( !_pCluster  ||  _nIndex == _nClusterSize )   // create a new cluster
  {
    ECluster aCluster = new TElm[ _nClusterSize ];
    _lCluster.insertAtEnd( aCluster );
    _pCluster = aCluster;
    _nIndex = 0;
  }

  _pCluster[ _nIndex++ ] = elm;
}

/** Pop an element from the stack.
  * Pop an element from the stack. The Stack must not be empty.
  * @param elm poped element.
  * @exception EXCStackEmpty raise if the stack is empty.
  */
template<class TElm, class TElmArg>
void EClusterStack<TElm,TElmArg>::pop( TElm *elm )
{
  if ( !_pCluster )
    ETHROW( EXCStackEmpty() );

  if ( _nIndex == 0 )   // delete cluster and get previous one.
  {
    if ( _lCluster.count() )
      _lCluster.deleteLast();

    if ( !_lCluster.count() )    // no more cluster => stack empty
    {
      ETHROW( EXCStackEmpty() );
    }
    else
    {
      _pCluster = _lCluster.at( _lCluster.count() -1 );
      _nIndex = _nClusterSize;
    }
  }

  *elm = _pCluster[ --_nIndex ];
}

template<class TElm, class TElmArg>
inline TElm EClusterStack<TElm,TElmArg>::pop()
{
  TElm elmTmp;
  pop( &elmTmp );
  return elmTmp;
}


template<class TElm, class TElmArg>
TElm & 
EClusterStack<TElm,TElmArg>::top()
{
  if ( !_pCluster )
    ETHROW( EXCStackEmpty() );
  
  ECluster pCluster = _pCluster;
  SL nIndex = _nIndex;
  if ( nIndex == 0 )   // delete cluster and get previous one.
  {
    if ( _lCluster.count() <= 1 )    // no more cluster => stack empty
    {
      ETHROW( EXCStackEmpty() );
    }
    else
    {
      pCluster = _lCluster.at( _lCluster.count() -2 );
      nIndex = _nClusterSize;
    }
  }

  return pCluster[ nIndex -1 ];
}

/** Clear the stack.
  * Clear the stack, therefore it is empty.
  */
template<class TElm, class TElmArg>
void EClusterStack<TElm,TElmArg>::clear()
{
  while ( _lCluster.count() )
    _lCluster.deleteFirst();

  _pCluster = ENIL;
  _nIndex = 0;
}


#define ESTACK_HPP_INCLUDED
#endif
