// ###########################################################################
// File: ESTREAM.HPP
// (c)Copyright 04/1998, Lepilleur Baptiste (Aka Elyon from Zuul Design)
// ###########################################################################


#ifndef ESTREAM_HPP_INCLUDED


// Includes:
// #####################Last change: 04/04/98 16h13mn#########################

#include "eiodevic.hpp"


// Constantes:
// #####################Last change: 04/04/98 16h13mn#########################

// if set then allow printable stream for debugging.
#ifndef NDEBUG
#define PRINTABLE_STREAM 1
#endif


// Macros:
// #####################Last change: 04/04/98 16h13mn#########################


// Types:
// #####################Last change: 04/04/98 16h13mn#########################

/** A Generic stream.
 */
class EStream
{
public:

    /// Byte order for storage. Default is big endian.
    enum ByteOrder
    {
      littleEndian =0,
      bigEndian
    };


    /// Constructs a data stream with no device.
    EStream();

    /** Constructs a data stream with a device.
     * Constructs a data stream for device \a device.
     * @param pDevice device used by the data stream.
     */
    EStream( EIODevice *pDevice );

    /// Destructor.
    ~EStream();

    /** Set device used by the data stream.
     * Set the device used by the data stream.
     * @param pDevice device used by the data stream.
     */
    virtual void setDevice( EIODevice *pDevice );

    /** Return the device.
     * @return device used by the data stream.
     */
    EIODevice *device() const;

    /** Unset device used by the data stream.
     * Unset the device used by the data stream.
     */
    void unsetDevice();

    /** Set the byte order.
     * Set the byte order to \a byteorder.
     * @param byteorder byte order used for data storage.
     */
    void setByteOrder( ByteOrder byteOrder );

    /** Return the byte order.
     * @return byte order.
     */
    EStream::ByteOrder byteOrder() const;

#ifdef PRINTABLE_STREAM
    /** Set printable stream.
     * If \a printableStream is true then the stream is considered as a text
     * stream. Allow value are written in text format.
     * @param printableStream if true then the stream is printable.
     */
    void setPrintableStream( bool bPrintableStream =true );

    /** Test if the stream is printable.
     * @return true if the stream is printable.
     */
    bool isPrintable() const;
#endif

    /// Write the last bit if needed.
    virtual void flush() {};

protected:

    /** Test if storing need byte swapping.
     * @return true if the storing need byte swapping.
     */
    bool needSwap() const;

    /// write an integer as text
    void writeInteger( SL v );

    /// read an integer from text
    SL readInteger();

    /// write a double as text
    void writeDouble( double v );

    /// read a double from text
    double readDouble();

    /** Get a printable byte from the stream.
     * @param on return, byte read.
     * @return reference on this stream.
     */
    EStream &getPrintableByte( SB& v );

    /** Put a printable byte in the stream.
     * @param v byte to put.
     * @return reference on this stream.
     */
    EStream &putPrintableByte( SB v );

private:
    /// initialisation
    void init();


    /// device used by the data stream.
    EIODevice *_pDevice;

    /// Byte order.
    ByteOrder _nOrder;

    /// Need to swap bytes for byte order if true.
    bool _bNeedSwap;

#ifdef PRINTABLE_STREAM
    /// Printable stream: text stream.
    bool _bIsPrintable;
#endif
};


// Inlines:
// #####################Last change: 04/04/98 16h13mn#########################

/** Return the device.
  * @return device used by the data stream.
  */
inline EIODevice *EStream::device() const
{
  return _pDevice;
}

/** Return the byte order.
  * @return byte order.
  */
inline EStream::ByteOrder EStream::byteOrder() const
{
  return _nOrder;
}

#ifdef PRINTABLE_STREAM
/** Set printable stream.
  * If \a printableStream is true then the stream is considered as a text
  * stream. Allow value are written in text format.
  * @param printableStream if true then the stream is printable.
  */
inline void EStream::setPrintableStream( bool bPrintableStream )
{
  _bIsPrintable = bPrintableStream;
}

/** Test if the stream is printable.
  * @return true if the stream is printable.
  */
inline bool EStream::isPrintable() const
{
  return _bIsPrintable;
}
#endif

/** Test if storing need byte swapping.
  * @return true if the storing need byte swapping.
  */
inline bool EStream::needSwap() const
{
  return _bNeedSwap;
}

// Prototypes:
// #####################Last change: 04/04/98 16h13mn#########################



#define ESTREAM_HPP_INCLUDED
#endif


// ########################### EOF ESTREAM.HPP ###############################
// ########################### EOF ESTREAM.HPP ###############################
// ########################### EOF ESTREAM.HPP ###############################

