// TtrAutoDetectCtx.cpp: implementation of the TtrAutoDetectCtx class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ttr.h"
#include "TtrAutoDetectCtx.h"
#include "TtrTextBlock.h"
#include "TtrFormatSetting.h"


/* Heuristic rules:
 * + Strip one blank line every other line if >45 % of blank line in text and
 *   they are every other line.
 *
 * + Line Is Indented & left margin: 2 convergences detected for indent size,
 *   with one where most of the line begins with upper case.
 *
 * + Previous Line Is Blank: 95% of the lines following a blank line do not start
 *   with a lower case character.
 *
 * - EOS : < 5%
 * - Right Margin: < 5%  but convergence between line that would be indented with
 *   EOS is low ( < 75% ).
 *
 * + Script mode detection: High number of lines containing ':'.
 */





#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TtrAutoDetectCtx::TtrAutoDetectCtx()
{
}


TtrAutoDetectCtx::~TtrAutoDetectCtx()
{
}


void 
TtrAutoDetectCtx::analyse( const TtrTextBlock &block )
{
  if ( !block.isValid() )
    return;

  _nLineCount = block.range().lineCount();

  for ( SL nLineNo =block.range().firstLine(); 
        nLineNo <= block.range().lastLine();
        ++nLineNo )
  {
    const TtrTextLine &line = block.text().line( nLineNo );

    SL nLineIndent = line.indentCount();

    // Line is indented / left margin
    IndentLineInfo *pIndentOcc = _hashIndentOcc.lookUp( nLineIndent );
    if ( pIndentOcc )
    {
      pIndentOcc->_nCount++;
      if ( isLowerCaseLine( line ) )
        pIndentOcc->_nCountStartWithLowerCase++;
    }
    else
      _hashIndentOcc.add( nLineIndent, 
                          IndentLineInfo( nLineIndent, 
                                          1, 
                                          isLowerCaseLine( line ) ) );


    // Script mode detection.
    if ( line.contains( _T( ':' ) ) )
      ++_nCountLineContainColon;


    // Every other line is a blank line.
    // Detects pattern: blank/text/blank and text/blank/text
    if ( line.isBlank()  &&  
         !block.text().line( nLineNo +1 ).isBlank()  &&
         block.text().line( nLineNo +2 ).isBlank() )
      ++_nCountHalfBlankPattern;
    else if ( !line.isBlank()  &&
         block.text().line( nLineNo +1 ).isBlank()  &&
         !block.text().line( nLineNo +2 ).isBlank() )
      ++_nCountHalfBlankPattern;
  }
}


void 
TtrAutoDetectCtx::resetAnalyse()
{
  _hashIndentOcc.removeAll();

  _nCountLineContainColon = 0;
  _nCountHalfBlankPattern = 0;
}


bool 
TtrAutoDetectCtx::isLowerCaseLine( const TtrTextLine &line )
{
  return ::islower( line.firstNonWhiteChar() )  &&
                    line.firstNonWhiteChar() != 0;
}


void 
TtrAutoDetectCtx::guessSetting( TtrFormatSetting &setting )
{
  if ( _nLineCount <= 0 )
    return;

  double fRatioScript = _nCountLineContainColon / _nLineCount;

  if ( fRatioScript > .05 )     // More than 5% of lines containing ':'
  {
    setupScriptSetting( setting );
    return;
  }

  makeSortedLineInfo();


}


void 
TtrAutoDetectCtx::setupScriptSetting( TtrFormatSetting &setting )
{
  setting.setFormatIndentBlankLine( true );
  setting.setFormatModeEos( true );
  setting.setFormatIndentIndentedLine( false );
  setting.setFormatIndentPreviousLineIsBlank( true );
  setting.setFormatIndentSeparator( true );
  setting.setFormatIndentContainDoubleDot( true );
  setting.setFormatIndentBeginWith( true );
  setting.setFormatUseEnclosure( true );
  setting.setFormatUseLeftMargin( false );
  setting.setFormatUseRightMargin( false );
  setting.setFormatNotLowerCaseLine( true );
}


void 
TtrAutoDetectCtx::makeSortedLineInfo()
{
  HashIndentOcc::EIterator it( _hashIndentOcc );
  for ( it.begin(); !it.isEnd(); it.next() )
    _sortedLineInfo.add( it.item() );
}
