// //////////////////////////////////////////////////////////////////////////
// Header file TtrFmtTextBuilder.h for class TtrFmtTextBuilder
// (c)Copyright 2000, Baptiste Lepilleur.
// Created: 2000/09/16
// //////////////////////////////////////////////////////////////////////////
#ifndef TTRFMTTEXTBUILDER_H
#define TTRFMTTEXTBUILDER_H

#include "TtrTextReformatterBuilder.h"
#include "TtrFmtElt.h"
#include "TtrTextLineFormat.h"
class TtrFmtText;
class TtrWriterSetting;

/*! \class TtrFmtTextBuilder
 * \brief This class is a builder to constructs formatted text.
 */
class TtrFmtTextBuilder : public TtrTextReformatterBuilder
{
  friend class TtrUtFmtTextBuilder;
public:
  /*! Constructs a TtrFmtTextBuilder object.
   * @param text Text that is built.
   * @param setting Writing setting.
   */
  TtrFmtTextBuilder( TtrFmtText &text,
                     const TtrWriterSetting &setting );

  /*! Destructor.
   */
  virtual ~TtrFmtTextBuilder();

  /** Changes the current block level.
   * @param nBlockLevel Depth of the next block.
   * @param strLevelPrefix Prefix string for the next block.
   */
  virtual void changeLevel( SL nBlockLevel,
                            const EString &strLevelPrefix );

  /**  Starts a new block using the specified content and indentation.
   * @param strContent Content of the new block. The content is trimmed
   *                   before being used for the block.
   * @param nIndentCount Number of indent to apply at the beginning
   *                     of the block. If 0 then \a strIndentString
   *                     it used to make the indentation. Must be >=0.
   * @param strIndentString String used if \a nIndentCount is 0 to
   *                        indent the new block.
   * @param format Format of the new block.
   */
  virtual void newBlock( const EString &strContent,
                         SL nIndentCount,
                         const EString &strIndent,
                         const TtrTextLineFormat &format );

  /** Appends the specified text to the current block.
   *
   * The specified text is trimmed then appended to the current block
   * using the hyphen rule.
   *
   * newBlock() must be called before any call to appendToBlock().
   *
   * @param strTextToAppend Text to append to the current block.
   */
  virtual void appendToBlock( const EString &strTextToAppend );

  /** Ends the current block.
   */
  virtual void endBlock();

private:
  /*! Prevents the use of the copy constructor.
   */
  TtrFmtTextBuilder( const TtrFmtTextBuilder &copy );

  /*! Prevents the use of the copy operator.
   */
  TtrFmtTextBuilder &operator =( const TtrFmtTextBuilder &copy );

  bool isCurrentBlockValid() const;

  void addParagraphToText();
  void addBlankToText();
  void makeCurrentBlockInvalid();
  bool isTextEmpty() const;

private:
  struct BlockDepthInfo
  {
    SL _nDepth;
    EString _strPrefix;
  };

  struct BlockInfo
  {
    SL _nIndentCount;
    EString _strIndent;
    EString _strContent;
    TtrTextLineFormat _format;
  };

private:
   TtrFmtText *_pText; 

   BlockDepthInfo _infoBlockDepth;
   BlockInfo _infoBlock;

   /*! Indicates if line ending with an hyphen ( '-' ) must be concatened
    * with the following line once the hyphen has been removed.
    */
   bool _bRemoveEndingHyphen;

   /*! Indicates if a blank line must be added after each paragraph.
    */
   bool _bJumpOneLine;
};



// Inlines methods for TtrFmtTextBuilder:
// --------------------------------------



#endif  // TTRFMTTEXTBUILDER_H
