// //////////////////////////////////////////////////////////////////////////
// Implementation file TtrTextReformatter.cpp for class TtrTextReformatter
// (c)Copyright 2000, Baptiste Lepilleur.
// Created: 2000/09/17
// //////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "TtrTextReformatter.h"
#include "TtrText.h"
#include "TtrTextLine.h"
#include "TtrFmtTextBuilder.h"
#include "EwgProgressInfo.h"


// ////////////////////////////////////////////////////////////////////
// TtrTextReformatter::BuildContext
// ////////////////////////////////////////////////////////////////////
TtrTextReformatter::BuildContext::BuildContext( 
                                      SL nBlockLevel ) :
      _nBlockLevel( nBlockLevel )
{
}




// ////////////////////////////////////////////////////////////////////
// TtrTextReformatter
// ////////////////////////////////////////////////////////////////////

TtrTextReformatter::TtrTextReformatter( TtrTextReformatterBuilder &builder ) :
    _pBuilder( &builder )
{
}


TtrTextReformatter::~TtrTextReformatter()
{
}


void 
TtrTextReformatter::reformat( const TtrText &text,           
                              EwgProgressInfo &progress )
{
  EASSERT( _pBuilder );

  progress.setProgressRange( 0, text.nbLine() );
  progress.startProgress();

  if ( text.nbLine() > 0 )
    buildTextBlocks( text, progress);

  progress.stopProgress();
}


void 
TtrTextReformatter::buildTextBlocks( const TtrText &text,           
                                     EwgProgressInfo &progress )
{
  BuildContext buildContext;
  changeLevel( text.line( 0 ), buildContext );
  startNewBlock( text.line( 0 ) );

  for ( int nLineNo =1; nLineNo < text.nbLine(); ++nLineNo )
  {
    updateProgress( progress, nLineNo );

    const TtrTextLine &line = text.line( nLineNo );
    if ( isLineChangingLevel( line, buildContext ) )
    {
      changeLevel( line, buildContext );
      startNewBlock( line );
    }
    else if ( isLineStartingNewBlock( line ) )
      startNewBlock( line );
    else
      continueBlock( line );
  }
  endBlock();
}


void 
TtrTextReformatter::updateProgress( EwgProgressInfo &progress,
                                   SL nLineNo )
{
  if ( (nLineNo & 127) == 0 )
    progress.setProgressValue( nLineNo );
}


void 
TtrTextReformatter::changeLevel( const TtrTextLine &line,
                                 BuildContext &buildContext )
{
  SL nBlockLevel = line.indentLength();

  builder().changeLevel( nBlockLevel, 
                         line.text().left( nBlockLevel ) );

  buildContext._nBlockLevel = nBlockLevel;
}


bool 
TtrTextReformatter::isLineChangingLevel( 
                       const TtrTextLine &line,
                       const BuildContext &buildContext ) const
{
  return line.indentLength() != buildContext._nBlockLevel;
}


void 
TtrTextReformatter::startNewBlock( const TtrTextLine &line )
{
  if ( line.indentLevel() == 0 )
    startNewBlockPreservingIndentation( line.content(), line.format() );
  else
  {
    SL nBoundedIndentLevel = emax( 1, line.indentLevel() );
    startNewIndentedBlock( line.content(), 
                           nBoundedIndentLevel, 
                           line.format() );
  }
}


void 
TtrTextReformatter::continueBlock( const TtrTextLine &line )
{
  builder().appendToBlock( line.content() );
}


void 
TtrTextReformatter::endBlock()
{
  builder().endBlock();
}


void 
TtrTextReformatter::startNewBlockPreservingIndentation( 
                        const EString &strLineContent,
                        const TtrTextLineFormat &format )
{
  EString strContent( strLineContent );
  strContent.trimLeft();

  SL nIndentLength = strLineContent.length() - strContent.length();
  EString strIndent( strLineContent.left( nIndentLength ) );

  builder().newBlock( strLineContent, 0, strIndent, format );
}


void
TtrTextReformatter::startNewIndentedBlock( 
                       const EString &strLineContent,
                       SL nIndentCount,
                       const TtrTextLineFormat &format )
{
  EString strTrimmedContent( strLineContent );
  builder().newBlock( strTrimmedContent, 
                      nIndentCount, 
                      EString::nullString,
                      format );
}


bool 
TtrTextReformatter::isLineStartingNewBlock( const TtrTextLine &line ) const
{
  return line.indentLevel() >= 0;
}


TtrTextReformatterBuilder &
TtrTextReformatter::builder() const
{
  EASSERT( _pBuilder != NULL );
  return *_pBuilder;
}


