// //////////////////////////////////////////////////////////////////////////
// Header file TtrTextReformatter.h for class TtrTextReformatter
// (c)Copyright 2000, Baptiste Lepilleur.
// Created: 2000/09/17
// //////////////////////////////////////////////////////////////////////////
#ifndef TTRTEXTREFORMATTER_H
#define TTRTEXTREFORMATTER_H


class TtrText;
class TtrTextLine;
class EwgProgressInfo;
class TtrTextReformatterBuilder;
class TtrTextLineFormat;


// ////////////////////////////////////////////////////////////////////
// TtrTextReformatter
// ////////////////////////////////////////////////////////////////////

/*!
 * \brief This class implements a Finite StateMachine to write the text.
 */
class TtrTextReformatter
{
  friend class TtrUtReformatter;
public:
  /*! Constructs a text reformatter for the specified builder.
   * @param builder Builder that is used to build the formatted text.
   */
	TtrTextReformatter( TtrTextReformatterBuilder &builder );

  /*! Destructor.
   */
	virtual ~TtrTextReformatter();

  /*! Reformats the specified text.
   *
   * Put the paragraphs of \a text back together using the "indented"
   * and "glued" indicator of each line.
   *
   * @param text Text used for reformatting.
   * @param progress Used to show progress.
   */
  virtual void reformat( const TtrText &text,           
                         EwgProgressInfo &progress );

  /*! Returns the reformatted text builder.
   * @return Reference on the reformatted text builder.
   */
  TtrTextReformatterBuilder &builder() const;

private:
  struct BuildContext
  {
    BuildContext( SL nBlockLevel =0 );

    SL _nBlockLevel;
  };

private:

  /*! Builds each block of the specified text.
   * @param text Text used for reformatting. The text must contains
   *             at least one line. 
   * @param progress Progress indicator.
   */
  void buildTextBlocks( const TtrText &text,           
                        EwgProgressInfo &progress );

  /*! Informs the builder of a level change and updates the build context.
   *
   * A level can be though of different level of indentation, but there
   * is also a "indent string" that represents that indentation. 
   * In e-mail, when answering a message, the sender's message is prefixed
   * using ">". Each occurence of ">" at the beginning of the line
   * represents a level.
   *
   * @param line [In] Line which is used to extract "level" information.
   * @param buildContext [Out] Build context to update for detecting
   *                     future level change. May be uninitialized.
   * @see isLineStartingNewBlockLevel().
   */
  void changeLevel( const TtrTextLine &line,
                    BuildContext &buildContext );

  /*! Tests if the specified line begins a new level.
   * @param line Line used to test if there is 
   * @param buildContext Build context used to detect change in the level.
   */
  bool isLineChangingLevel( const TtrTextLine &line,
                            const BuildContext &buildContext ) const;

  /*! Builds a new block using the specified line.
   * @param line Line used to build the new block.
   */
  void startNewBlock( const TtrTextLine &line );

  /*! Appends the specified line to the current block.
   * @param line Line to append to the current block.
   */
  void continueBlock( const TtrTextLine &line );

  /*! Builds a new block using the specified line while preserving its indentation.
   * @param strLineContent Content of the line (line without "level" string).
   * @param format Format associated to the line.
   */
  void startNewBlockPreservingIndentation( const EString &strLineContent ,
                                           const TtrTextLineFormat &format );

  /*! Builds a new indented block using the specified indentation.
   * @param strLineContent Content of the line (line without "level" string).
   * @param nIndentCount Number of indentations to produce in the reformatted text.
   * @param format Format associated to the line.
   */
  void startNewIndentedBlock( const EString &strLineContent,
                              SL nIndentCount,
                              const TtrTextLineFormat &format );

  /*! Ends the current block.
   */
  void endBlock();

  /*! Tests if the specified line starts a new block (is not glued).
   * @param line Line which is tested.
   * @return \c true if the line starts a new block, otherwise returns
   *         \c false (the line is glued: its indent < 0).
   */
  bool isLineStartingNewBlock( const TtrTextLine &line ) const;

  /*! Reports progress to the progress indicator.
   * @param progress Progress indicator.
   * @param nLineNo Number of the line being analysed.
   */
  void updateProgress( EwgProgressInfo &progress,
                       SL nLineNo );
private:
  /*! Pointer on the formatted text builder.
   * @supplierCardinality 1
   */
  TtrTextReformatterBuilder *_pBuilder;
};




// implemention of TtrTextReformatter 
// ////////////////////////////////////////////////////////////////////


#endif  // TTRTEXTREFORMATTER_H
