// //////////////////////////////////////////////////////////////////////////
// Implementation file TtrUtReformatter.cpp for class TtrUtReformatter
// (c)Copyright 2000, Baptiste Lepilleur.
// Created: 2001/01/17
// //////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "TtrUtReformatter.h"
#include "EwgProgressInfo.h"
#include "TtrFmtBlank.h"
#include "TtrFmtEltVisitor.h"
#include "TtrFmtPara.h"
#include "TtrFmtText.h"
#include "TtrFmtTextBuilder.h"
#include "TtrFmtTextComparator.h"
#include "TtrFmtTextDumper.h"
#include "TtrTextReformatter.h"
#include "TtrGlobalSetting.h"
#include "TtrTextRaw.h"

CPPUNIT_TEST_SUITE_REGISTRATION( TtrUtReformatter );


class TtrUtReformatter::TtrTextBuilder
{
public:
  TtrTextBuilder( TtrText &text ) : _text( text ) {
  }

  void addIndented( const EString &strText, SL nPrefixLength = 0 )
  {
    add( strText, nPrefixLength, 1 );
  }

  void addGlued( const EString &strText, SL nPrefixLength = 0 )
  {
    add( strText, nPrefixLength, -1 );
  }

  void addNormal( const EString &strText, SL nPrefixLength = 0 )
  {
    add( strText, nPrefixLength, 0 );
  }

protected:
  void add( const EString &strText, SL nPrefixLength, SL nIndentLevel ) 
  {
    TtrTextLine *pLine = new TtrTextLine( strText );
    pLine->setIndentLength( nPrefixLength );
    pLine->setIndentLevel( nIndentLevel );
    _text.insertLine( pLine );
  }

  TtrText &_text;
};


TtrUtReformatter::TtrUtReformatter()
{
}


TtrUtReformatter::~TtrUtReformatter()
{
}


void 
TtrUtReformatter::setUp()
{
  _nExpectedCallIndex = 0;
  _formatNone = TtrTextLineFormat();
  _formatCenter = TtrTextLineFormat().centerOn();
  _pReformatter = new TtrTextReformatter( *this );
}


void 
TtrUtReformatter::tearDown()
{
  _vCalls.reset();
  delete _pReformatter;
}


void
TtrUtReformatter::testBuildTextBlocks()
{
  TtrTextRaw text;
  TtrTextBuilder builder( text );
  builder.addIndented( "  Indented" );
  builder.addGlued( "glued" );
  builder.addGlued( "glued2" );
  builder.addIndented( ">  Indented & prefixed", 1 );
  builder.addGlued( "> glued & prefixed", 1 );
  builder.addNormal( "\tPreserving indent", 0 );

  expectCall( new ChangeLevelParams( 0, "" ) );
  expectCall( new NewBlockParams( "Indented", 1, "", _formatNone ) );
  expectCall( new AppendToBlockParams( "glued" ) );
  expectCall( new AppendToBlockParams( "glued2" ) );
  expectCall( new NewBlockParams( "  Indented & prefixed", 1, "", _formatNone ) );
  expectCall( new AppendToBlockParams( " glued & prefixed" ) );
  expectCall( new NewBlockParams( "Preserving indent", 0, "\t", _formatNone ) );
  expectCall( new EndBlockParams() );

  // something odd there...
}


void 
TtrUtReformatter::testChangeLevel()
{
  TtrTextReformatter reformatter( *this );
  TtrTextReformatter::BuildContext context;

  TtrTextLine line1( ">>Text" );
  line1.setIndentLength( 2 );
  expectCall( new ChangeLevelParams( 2, ">>" ) );
  reformatter.changeLevel( line1, context );
  GAIACRTN_ASSERT_EQUAL( 2, context._nBlockLevel );

  TtrTextLine line2( "> Text" );
  line2.setIndentLevel( 1 );
  expectCall( new ChangeLevelParams( 0, "" ) );
  reformatter.changeLevel( line2, context );
  GAIACRTN_ASSERT_EQUAL( 0, context._nBlockLevel );
}


void 
TtrUtReformatter::testIsGluedAndPrefixedLineChangingLevel()
{
  TtrTextReformatter::BuildContext context;
  TtrTextLine line( ">glued and prefixed line." );
  line.setIndentLevel( -1 );
  line.setIndentLength( 1 );
  GAIACRTN_ASSERT( _pReformatter->isLineChangingLevel( line , context ) );
}


void
TtrUtReformatter::testIsIndentedLineChangingLevel()
{
  TtrTextReformatter::BuildContext context;
  TtrTextLine line( "  An indented line." );
  line.setIndentLevel( 1 );
  GAIACRTN_ASSERT( !_pReformatter->isLineChangingLevel( line , context ) );
}


void
TtrUtReformatter::testIsIndentedLineChangingLevelTo0()
{
  TtrTextReformatter::BuildContext context( 1 );
  TtrTextLine line( "  An indented line." );
  line.setIndentLevel( 1 );
  GAIACRTN_ASSERT( _pReformatter->isLineChangingLevel( line , context ) );
}


void 
TtrUtReformatter::testStartNewBlock()
{
  TtrTextLine line1( " \t Original indent" );
  line1.setIndentLevel( 0 );
  TtrTextLine line2( "\tIndented line" );
  line2.setIndentLevel( 1 );

  expectCall( new NewBlockParams( line1.text(), 0, " \t ", _formatNone ) );
  expectCall( new NewBlockParams( line2.text(), 1, "", _formatNone ) );
  expectCall( new NewBlockParams( line2.text(), 3, "", _formatNone ) );
  expectCall( new NewBlockParams( line2.text(), 1, "", _formatNone ) );

  TtrTextReformatter reformatter( *this );
  reformatter.startNewBlock( line1 );
  reformatter.startNewBlock( line2 );
  line2.setIndentLevel( 3 );
  reformatter.startNewBlock( line2 );
  line2.setIndentLevel( -1 );
  reformatter.startNewBlock( line2 );

  checkNoRemainingCall();
}


void 
TtrUtReformatter::testContinueBlock()
{
  TtrTextLine line1( ">> \t Glued prefixed indented line." );
  line1.setIndentLevel( -1 );
  line1.setIndentLength( 2 );
  expectCall( new AppendToBlockParams( " \t Glued prefixed indented line." ) );

  TtrTextLine line2( " Glued line." );
  line2.setIndentLevel( -1 );
  expectCall( new AppendToBlockParams( line2.text() ) );

  TtrTextReformatter reformatter( *this );
  reformatter.continueBlock( line1 );
  reformatter.continueBlock( line2 );

  checkNoRemainingCall();
}


void 
TtrUtReformatter::testStartNewBlockPreservingIndentation()
{
  EString strLine1( "Line1" );
  EString strLine2( "   Line2   " );
  EString strLine3( "\t\t Line2   " );
  expectCall( new NewBlockParams( strLine1, 0, "", _formatNone ) );
  expectCall( new NewBlockParams( strLine2, 0, "   ", _formatCenter ) );
  expectCall( new NewBlockParams( strLine3, 0, "\t\t ", _formatNone ) );

  TtrTextReformatter reformatter( *this );
  reformatter.startNewBlockPreservingIndentation( strLine1, _formatNone );
  reformatter.startNewBlockPreservingIndentation( strLine2, _formatCenter );
  reformatter.startNewBlockPreservingIndentation( strLine3, _formatNone );

  checkNoRemainingCall();
}


void 
TtrUtReformatter::testStartNewIndentedBlock()
{
  EString strLine1( "Line1" );
  EString strLine2( "   Line2   " );
  expectCall( new NewBlockParams( strLine1, 1, "", _formatNone ) );
  expectCall( new NewBlockParams( strLine2, 1, "", _formatCenter ) );
  expectCall( new NewBlockParams( strLine2, 2, "", _formatCenter ) );

  TtrTextReformatter reformatter( *this );
  reformatter.startNewIndentedBlock( strLine1, 1, _formatNone );
  reformatter.startNewIndentedBlock( strLine2, 1, _formatCenter );
  reformatter.startNewIndentedBlock( strLine2, 2, _formatCenter );

  checkNoRemainingCall();
}


void 
TtrUtReformatter::testEndBlock()
{
  expectCall( new EndBlockParams() );  

  TtrTextReformatter reformatter( *this );
  reformatter.endBlock();

  checkNoRemainingCall();
}


void 
TtrUtReformatter::testIsIndentedLineStartingNewBlock()
{
  TtrTextLine line( "    Indented line." );
  line.setIndentLevel( 1 );
  GAIACRTN_ASSERT( _pReformatter->isLineStartingNewBlock( line ) );
}


void 
TtrUtReformatter::testIsIndentedLineWithPrefixStartingNewBlock()
{
  TtrTextLine line( ">> Indented and prefixed line." );
  line.setIndentLevel( 1 );
  line.setIndentLength( 2 );
  GAIACRTN_ASSERT( _pReformatter->isLineStartingNewBlock( line ) );
}


void 
TtrUtReformatter::testIsPrefixedLineStartingNewBlock()
{
  TtrTextLine line( ">>Prefixed line." );
  line.setIndentLength( 2 );
  GAIACRTN_ASSERT( _pReformatter->isLineStartingNewBlock( line ) );
}


void 
TtrUtReformatter::testIsGluedLineStartingNewBlock()
{
  TtrTextLine line( "glued line." );
  line.setIndentLevel( -1 );
  GAIACRTN_ASSERT( !_pReformatter->isLineStartingNewBlock( line ) );
}


void 
TtrUtReformatter::testIsGluedLineWithPrefixStartingNewBlock()
{
  TtrTextLine line( ">glued line." );
  line.setIndentLevel( -1 );
  line.setIndentLength( 1 );
  GAIACRTN_ASSERT( !_pReformatter->isLineStartingNewBlock( line ) );
}


void 
TtrUtReformatter::expectCall( Root *pCall )
{
  _vCalls.append( pCall );
}


void 
TtrUtReformatter::checkNoRemainingCall()
{
  GAIACRTN_ASSERT_EQUAL( _nExpectedCallIndex, _vCalls.count() );
}


void 
TtrUtReformatter::changeLevel( SL nBlockLevel,
                               const EString &strLevelPrefix )
{
  ChangeLevelParams *pCall;
  getNextCall( &pCall );
  GAIACRTN_ASSERT_EQUAL( pCall->_nBlockLevel, nBlockLevel );
  GAIACRTN_ASSERT_EQUAL( pCall->_strLevelPrefix, strLevelPrefix );
}


void 
TtrUtReformatter::newBlock( const EString &strContent,
                            SL nIndentCount,
                            const EString &strIndent,
                            const TtrTextLineFormat &format )
{
  NewBlockParams *pCall;
  getNextCall( &pCall );
  GAIACRTN_ASSERT_EQUAL( pCall->_strContent, strContent );
  GAIACRTN_ASSERT_EQUAL( pCall->_nIndentCount, nIndentCount );
  GAIACRTN_ASSERT_EQUAL( pCall->_strIndent, strIndent );
  GAIACRTN_ASSERT_EQUAL( pCall->_format, format );
}


void 
TtrUtReformatter::appendToBlock( const EString &strTextToAppend )
{
  AppendToBlockParams *pCall;
  getNextCall( &pCall );
  GAIACRTN_ASSERT_EQUAL( pCall->m_strTextToAppend, strTextToAppend );
}


void 
TtrUtReformatter::endBlock()
{
  EndBlockParams *pCall;
  getNextCall( &pCall );
}

