// //////////////////////////////////////////////////////////////////////////
// Implementation file TtrWriterWrappedTxt.cpp for class TtrWriterWrappedTxt
// (c)Copyright 2000, Baptiste Lepilleur.
// Created: 2000/09/27
// //////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "TtrWriterWrappedTxt.h"
#include "TtrWriterSetting.h"
#include "TtrFmtBlank.h"
#include "TtrFmtPara.h"
#include "TtrTagDef.h"
#include <string.h>
#include <stdlib.h>


TtrWriterWrappedTxt::TtrWriterWrappedTxt( ETextStream &rstreamOut,
                                          const TtrWriterSetting &setting ) :
  _rstreamOut( rstreamOut ),
  _nWrappingCol( setting.getWriterWrappingCol() )
{
  setup( setting );
}


TtrWriterWrappedTxt::~TtrWriterWrappedTxt()
{
}


void 
TtrWriterWrappedTxt::setup( const TtrWriterSetting &setting )
{
  _bKeepOriginalIndent = ( setting.getWriterLineMode() == wslmIndentAsOriginal );
  
  if ( _bKeepOriginalIndent )
    _strIndentStep = "";
  else
  {
    bool bUseTab = ( setting.getWriterLineMode() == wslmIndentWithTab );
    _strIndentStep.setNull();
    _strIndentStep.append( bUseTab ? '\t' : ' ', 
                           setting.getWriterReindentSize() );
  }
}


void 
TtrWriterWrappedTxt::visitBlank( TtrFmtBlank &blank )
{
  SL nCount = blank.lineCount();
  while ( --nCount >= 0 )
  {
    stream()  <<  endl;
  }
}


void 
TtrWriterWrappedTxt::visitPara( TtrFmtPara &para )
{
  EString strPrefix = para.indent().indentString();
  strPrefix.trimRight();    // because the prefix includes indent
  EString strContent;
  if ( !_bKeepOriginalIndent )
  {
    strContent.append( _strIndentStep, para.indent().indentLevel() );
    if ( para.indent().indentLevel() > 0 )
    {
      EString strTrimmedContent( para.content() );
      strTrimmedContent.trimLeft();
      strContent += strTrimmedContent;
    }
    else
      strContent += para.content();
  }
  else
    strContent = para.content();

  
  if ( para.format().isCentered() )
    writeCenteredPara( strPrefix, strContent );
  else
    writeLeftAlignedPara( strPrefix, strContent );
}


void 
TtrWriterWrappedTxt::writeCenteredPara( const EString &strPrefix,
                                        const EString &strParagraph )
{
  SL nPrefixLength = computeRightMarginPos( strPrefix, 0, strPrefix.length(), 0 );
  SL nMaxContentLineLength = emax( 1, _nWrappingCol - nPrefixLength );

  Strings lines;
  breakParagraph( replaceTabWithSpace( strParagraph ),
                  lines, 
                  nMaxContentLineLength, 
                  0 );

  for ( int nIdx =0; nIdx < lines.count(); ++nIdx )
  {
    const EString &strLine = lines[ nIdx ];
    SL nPaddingLength = ( nMaxContentLineLength -
                          computeRightMarginPos( strLine, 
                                                 0, 
                                                 strLine.length(), 
                                                 0 ) ) /2;
    EString strPadding;
    strPadding.append( ' ', nPaddingLength );
    stream()  <<  strPrefix  <<  strPadding  <<  strLine << endl;
  }
}


void 
TtrWriterWrappedTxt::writeLeftAlignedPara( const EString &strPrefix,
                                           const EString &strParagraph )
{
  SL nPrefixLength = computeRightMarginPos( strPrefix, 0, strPrefix.length(), 0 );

  Strings lines;
  breakParagraph( strParagraph,
                  lines, 
                  _nWrappingCol, 
                  nPrefixLength );

  for ( int nIdx =0; nIdx < lines.count(); ++nIdx )
  {
    const EString &strLine = lines[ nIdx ];
    stream()  <<  strPrefix  <<  strLine << endl;
  }
}


void 
TtrWriterWrappedTxt::findBreakPos( const EString &strText,
                                   ArrayBreakPos &vBreakPos ) const
{
  if ( strText.isNull() )
  {
    vBreakPos.append( BreakPos( 0, 0 ) );
    return;
  }

  SL nLength = strText.length();

  SL nPos =0;
  while ( (nPos < nLength )  && 
          ::isspace( strText[nPos] ) )    // skip space
    ++nPos;

  while ( nPos < nLength )
  {
    while ( nPos < nLength  &&
            !::isspace( strText[ nPos ] ) )   // skip word
      ++nPos;

    SL nPosStart = nPos;
    while ( nPos < nLength  &&                // skip space
      ::isspace( strText[ nPos ] ) )
      ++nPos;

    vBreakPos.append( BreakPos( nPosStart, nPos - nPosStart ) );
  }
}


bool 
TtrWriterWrappedTxt::isWordChar( char c ) const
{
  return !::isspace( c );
}


SL 
TtrWriterWrappedTxt::computeRightMarginPos( const EString &strLine,
                                            SL nSliceFrom,
                                            SL nSliceTo,
                                            SL nLeftMarginPos ) const
{
  SL nCurrentPos = nLeftMarginPos;
  const SL nTabLength = 8;

  for ( SL nIdx = nSliceFrom; nIdx < nSliceTo; ++nIdx )
  {
    char c = strLine[ nIdx ];
    if ( c != '\t' )
      ++nCurrentPos;
    else
    {
      div_t divTab = ::div( nCurrentPos, nTabLength );
      nCurrentPos += nTabLength - divTab.rem;
    }
  }

  return nCurrentPos;
}


void 
TtrWriterWrappedTxt::breakParagraph( const EString &strParagraph, 
                                     Strings &lines, 
                                     int nLineLength,
                                     int nLeftMarginPos ) const
{
  ArrayBreakPos breaks;
  findBreakPos( strParagraph, breaks );
  
  int nLineStartPos =0;
  int nLineEndPos = breaks[0]._nPos;
  int nIdxBreak = 0;

  do
  {
    int nBreakPos = breaks[nIdxBreak]._nPos;
    if ( computeRightMarginPos( strParagraph,
                                nLineStartPos,
                                nBreakPos,
                                nLeftMarginPos ) > nLineLength )
    {
      lines.append( strParagraph.mid( nLineStartPos, nLineEndPos-nLineStartPos ) );
      if ( nLineEndPos >= strParagraph.length() )
        break;

      int nIdxBreakStart = emax(0, nIdxBreak-1);
      nLineStartPos = breaks[nIdxBreakStart]._nPos + breaks[nIdxBreakStart]._nLength;
      if ( nIdxBreak == 0 )
        ++nIdxBreak;
    }

    nLineEndPos = breaks[nIdxBreak++]._nPos;

    if ( nLineEndPos >= strParagraph.length() )
    {
      lines.append( strParagraph.mid( nLineStartPos, nLineEndPos-nLineStartPos ) );
      break;
    }
  }
  while ( true );
}


EString 
TtrWriterWrappedTxt::replaceTabWithSpace( const EString &strParagraph ) const
{
  EString strSpacized;
  SL nLastBreakPos =0;
  SL nIdx = 0;
  for ( ; nIdx < strParagraph.length(); ++nIdx )
  {
    char c = strParagraph[nIdx];
    if ( c < 32 )
    {
      strSpacized += strParagraph.mid( nLastBreakPos, nIdx - nLastBreakPos );
      strSpacized += " ";
      nLastBreakPos = nIdx+1;
    }
  }
 
  strSpacized += strParagraph.mid( nLastBreakPos, nIdx - nLastBreakPos );
  return strSpacized;
}
